@@ -7,6 +7,7 @@
 #ifdef HAVE_CONFIG_H
 #include "dgconfig.h"
 #endif
+#include "DataBuffer.hpp"
 #include "HTTPHeader.hpp"
 #include "OptionContainer.hpp"
 
@@ -35,7 +36,7 @@ extern OptionContainer o;
 // IMPLEMENTATION
 
 DataBuffer::DataBuffer()
-    : data(new char[1]), buffer_length(0), compresseddata(NULL), compressed_buffer_length(0), tempfilesize(0), dontsendbody(false), tempfilefd(-1), dm_plugin(NULL), timeout(20), bytesalreadysent(0), preservetemp(false)
+    : data(new char[1]), buffer_length(0), compresseddata(NULL), compressed_buffer_length(0), tempfilesize(0), dontsendbody(false), tempfilefd(-1), dm_plugin(NULL), timeout(20), bytesalreadysent(0), preservetemp(false), stimeout(20000)
 {
     data[0] = '\0';
 }
@@ -151,7 +152,7 @@ int DataBuffer::bufferReadFromSocket(Socket *sock, char *buffer, int size, int s
         gettimeofday(&nowadays, NULL);
         if (nowadays.tv_sec - starttime.tv_sec > timeout) {
 #ifdef DGDEBUG
-            std::cout << "buffered socket read more than timeout" << std::endl;
+            std::cout << "buffered socket read more than timeout" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
             return pos; // just return how much got so far then
         }
@@ -173,7 +174,7 @@ int DataBuffer::getTempFileFD()
     umask(0007); // only allow access to e2g user and group
     if ((tempfilefd = mkstemp(tempfilepatharray)) < 0) {
 #ifdef DGDEBUG
-        std::cerr << "error creating temp " << tempfilepath << ": " << strerror(errno) << std::endl;
+        std::cout << "error creating temp " << tempfilepath << ": " << strerror(errno) << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
         syslog(LOG_ERR, "Could not create temp file to store download for scanning: %s", strerror(errno));
         tempfilefd = -1;
@@ -209,7 +210,7 @@ bool DataBuffer::in(Socket *sock, Socket *peersock, HTTPHeader *requestheader, H
     for (std::deque<Plugin *>::iterator i = o.dmplugins_begin; i != o.dmplugins_end; i++) {
         if ((i + 1) == o.dmplugins_end) {
 #ifdef DGDEBUG
-            std::cerr << "Got to final download manager so defaulting to always match." << std::endl;
+            std::cout << "Got to final download manager so defaulting to always match." << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
             dm_plugin = (DMPlugin *)(*i);
             rc = dm_plugin->in(this, sock, peersock, requestheader, docheader, runav, headersent, &toobig);
@@ -217,7 +218,7 @@ bool DataBuffer::in(Socket *sock, Socket *peersock, HTTPHeader *requestheader, H
         } else {
             if (((DMPlugin *)(*i))->willHandle(requestheader, docheader)) {
 #ifdef DGDEBUG
-                std::cerr << "Matching download manager number: " << j << std::endl;
+                std::cout << "Matching download manager number: " << j << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
                 dm_plugin = (DMPlugin *)(*i);
                 rc = dm_plugin->in(this, sock, peersock, requestheader, docheader, runav, headersent, &toobig);
@@ -234,51 +235,54 @@ bool DataBuffer::in(Socket *sock, Socket *peersock, HTTPHeader *requestheader, H
 }
 
 // send the request body to the client after having been handled by a DM plugin
-void DataBuffer::out(Socket *sock) throw(std::exception)
+bool DataBuffer::out(Socket *sock) //throw(std::exception)
 {
     if (dontsendbody) {
 #ifdef DGDEBUG
-        std::cout << "dontsendbody true; not sending" << std::endl;
+        std::cout << "dontsendbody true; not sending" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
-        return;
+        return true;
     }
-    (*sock).readyForOutput(timeout); // exceptions on timeout or error
+    if (!(*sock).breadyForOutput(stimeout)) return false; // exceptions on timeout or error
 
     if (tempfilefd > -1) {
 // must have been too big for ram so stream from disk in blocks
 #ifdef DGDEBUG
-        std::cout << "Sending " << tempfilesize - bytesalreadysent << " bytes from temp file (" << bytesalreadysent << " already sent)" << std::endl;
+        std::cout << "Sending " << tempfilesize - bytesalreadysent << " bytes from temp file (" << bytesalreadysent << " already sent)" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
         off_t sent = bytesalreadysent;
         int rc;
 
         if (lseek(tempfilefd, bytesalreadysent, SEEK_SET) < 0)
-            throw std::runtime_error(std::string("Can't write to socket: ") + strerror(errno));
+            return false;
+//            throw std::runtime_error(std::string("Can't write to socket: ") + strerror(errno));
 
         while (sent < tempfilesize) {
             rc = readEINTR(tempfilefd, data, buffer_length);
 #ifdef DGDEBUG
-            std::cout << "reading temp file rc:" << rc << std::endl;
+            std::cout << "reading temp file rc:" << rc << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
             if (rc < 0) {
 #ifdef DGDEBUG
-                std::cout << "error reading temp file so throwing exception" << std::endl;
+                std::cout << "error reading temp file so throwing exception" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
-                throw std::exception();
+                return false;
+    //            throw std::exception();
             }
             if (rc == 0) {
 #ifdef DGDEBUG
-                std::cout << "got zero bytes reading temp file" << std::endl;
+                std::cout << "got zero bytes reading temp file" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
                 break; // should never happen
             }
             // as it's cached to disk the buffer must be reasonably big
-            if (!sock->writeToSocket(data, rc, 0, timeout)) {
-                throw std::runtime_error(std::string("Can't write to socket: ") + strerror(errno));
+            if (!sock->writeToSocket(data, rc, 0, stimeout)) {
+                return false;
+//                throw std::runtime_error(std::string("Can't write to socket: ") + strerror(errno));
             }
             sent += rc;
 #ifdef DGDEBUG
-            std::cout << "total sent from temp:" << sent << std::endl;
+            std::cout << "total sent from temp:" << sent << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
         }
         close(tempfilefd);
@@ -287,17 +291,24 @@ void DataBuffer::out(Socket *sock) throw(std::exception)
         unlink(tempfilepath.toCharArray());
     } else {
 #ifdef DGDEBUG
-        std::cout << "Sending " << buffer_length - bytesalreadysent << " bytes from RAM (" << buffer_length << " in buffer; " << bytesalreadysent << " already sent)" << std::endl;
+        std::cout << "Sending " << buffer_length - bytesalreadysent << " bytes from RAM (" << buffer_length << " in buffer; " << bytesalreadysent << " already sent)" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
         // it's in RAM, so just send it, no streaming from disk
         if (buffer_length != 0) {
-            if (!sock->writeToSocket(data + bytesalreadysent, buffer_length - bytesalreadysent, 0, timeout))
-                throw std::exception();
+            if (!sock->writeToSocket(data + bytesalreadysent, buffer_length - bytesalreadysent, 0, stimeout))
+                return false;
+          //      throw std::exception();
         } else {
-            if (!sock->writeToSocket("\r\n\r\n", 4, 0, timeout))
-                throw std::exception();
+            if (!sock->writeToSocket("\r\n\r\n", 4, 0, stimeout))
+                return false;
+           //     throw std::exception();
         }
+#ifdef DGDEBUG
+        std::cout << "Sent " << buffer_length - bytesalreadysent << " bytes from RAM (" << buffer_length  << ") Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+#endif
+     return true;
     }
+   return true;
 }
 
 // zlib decompression
@@ -307,7 +318,7 @@ void DataBuffer::zlibinflate(bool header)
         return; // it can't possibly be zlib'd
     }
 #ifdef DGDEBUG
-    std::cout << "compressed size:" << buffer_length << std::endl;
+    std::cout << "compressed size:" << buffer_length << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
 
 #if ZLIB_VERNUM < 0x1210
@@ -351,13 +362,13 @@ void DataBuffer::zlibinflate(bool header)
     if (err != Z_OK) { // was a problem so just return
         delete[] block; // don't forget to free claimed memory
 #ifdef DGDEBUG
-        std::cerr << "bad init inflate: " << err << std::endl;
+        std::cout << "bad init inflate: " << err << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
         return;
     }
     while (true) {
 #ifdef DGDEBUG
-        std::cerr << "inflate loop" << std::endl;
+        std::cout << "inflate loop" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
         err = inflate(&d_stream, Z_SYNC_FLUSH);
         bytesgot = d_stream.total_out;
@@ -366,7 +377,7 @@ void DataBuffer::zlibinflate(bool header)
             if (err != Z_OK) {
                 delete[] block;
 #ifdef DGDEBUG
-                std::cerr << "bad inflateEnd: " << d_stream.msg << std::endl;
+                std::cout << "bad inflateEnd: " << d_stream.msg << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
                 return;
             }
@@ -375,12 +386,12 @@ void DataBuffer::zlibinflate(bool header)
         if (err != Z_OK) { // was a problem so just return
             delete[] block; // don't forget to free claimed memory
 #ifdef DGDEBUG
-            std::cerr << "bad inflate: " << d_stream.msg << std::endl;
+            std::cout << "bad inflate: " << d_stream.msg << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
             err = inflateEnd(&d_stream);
             if (err != Z_OK) {
 #ifdef DGDEBUG
-                std::cerr << "bad inflateEnd: " << d_stream.msg << std::endl;
+                std::cout << "bad inflateEnd: " << d_stream.msg << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
             }
             return;
@@ -388,12 +399,12 @@ void DataBuffer::zlibinflate(bool header)
         if (bytesgot > o.max_content_filter_size) {
             delete[] block; // don't forget to free claimed memory
 #ifdef DGDEBUG
-            std::cerr << "inflated file larger than maxcontentfiltersize, not inflating further" << std::endl;
+            std::cout << "inflated file larger than maxcontentfiltersize, not inflating further" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
             err = inflateEnd(&d_stream);
             if (err != Z_OK) {
 #ifdef DGDEBUG
-                std::cerr << "bad inflateEnd: " << d_stream.msg << std::endl;
+                std::cout << "bad inflateEnd: " << d_stream.msg << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
             }
             return;
@@ -415,7 +426,7 @@ void DataBuffer::zlibinflate(bool header)
     compressed_buffer_length = buffer_length;
     buffer_length = bytesgot;
 #ifdef DGDEBUG
-    std::cout << "decompressed size: " << buffer_length << std::endl;
+    std::cout << "decompressed size: " << buffer_length << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
     data = new char[bytesgot + 1];
     data[bytesgot] = '\0';
@@ -428,19 +439,20 @@ struct newreplacement {
     int match;
     String replacement;
 };
-bool DataBuffer::contentRegExp(int filtergroup)
+bool DataBuffer::contentRegExp(FOptionContainer* &foc)
 {
 
 #ifdef DGDEBUG
-    std::cout << "Starting content reg exp replace" << std::endl;
+    std::cout << "Starting content reg exp replace" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
     bool contentmodified = false;
     unsigned int i;
     unsigned int j, k, m;
-    unsigned int s = (*o.fg[filtergroup]).content_regexp_list_comp.size();
+    unsigned int s = (*foc).content_regexp_list_comp.size();
     unsigned int matches;
     unsigned int submatch, submatches;
     RegExp *re;
+    RegResult Rre;
     String *replacement;
     unsigned int replen;
     int sizediff;
@@ -453,21 +465,21 @@ bool DataBuffer::contentRegExp(int filtergroup)
     std::queue<newreplacement *> matchqueue;
 
     for (i = 0; i < s; i++) {
-        re = &((*o.fg[filtergroup]).content_regexp_list_comp[i]);
-        if (re->match(data)) {
-            replacement = &((*o.fg[filtergroup]).content_regexp_list_rep[i]);
+        re = &((*foc).content_regexp_list_comp[i]);
+        if (re->match(data, Rre)) {
+            replacement = &((*foc).content_regexp_list_rep[i]);
             //replen = replacement->length();
-            matches = re->numberOfMatches();
+            matches = Rre.numberOfMatches();
 
             sizediff = 0;
             m = 0;
             for (j = 0; j < matches; j++) {
-                srcoff = re->offset(j);
-                matchlen = re->length(j);
+                srcoff = Rre.offset(j);
+                matchlen = Rre.length(j);
 
                 // Count matches for ()'s
                 for (submatches = 0; j + submatches + 1 < matches; submatches++)
-                    if (re->offset(j + submatches + 1) + re->length(j + submatches + 1) > srcoff + matchlen)
+                    if (Rre.offset(j + submatches + 1) + Rre.length(j + submatches + 1) > srcoff + matchlen)
                         break;
 
                 // \1 and $1 replacement
@@ -484,7 +496,7 @@ bool DataBuffer::contentRegExp(int filtergroup)
                         submatch = (*replacement)[++k] - '0';
                         // add submatch contents to replacement string
                         if (submatch <= submatches) {
-                            newrep->replacement += re->result(j + submatch).c_str();
+                            newrep->replacement += Rre.result(j + submatch).c_str();
                         }
                     } else {
                         // unescape \\ and \$, and add other non-backreference characters
@@ -496,7 +508,7 @@ bool DataBuffer::contentRegExp(int filtergroup)
                 matchqueue.push(newrep);
 
                 // update size difference between original and modified content
-                sizediff -= re->length(j);
+                sizediff -= Rre.length(j);
                 sizediff += newrep->replacement.length();
                 // skip submatches to next top level match
                 j += submatches;
@@ -511,13 +523,13 @@ bool DataBuffer::contentRegExp(int filtergroup)
             matches = m;
 
 #ifdef DGDEBUG
-            std::cout << "content matches:" << matches << std::endl;
+            std::cout << "content matches:" << matches << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
 #endif
             // replace top-level matches using filled-out replacement strings
             newreplacement *newrep;
             for (j = 0; j < matches; j++) {
                 newrep = matchqueue.front();
-                nextoffset = re->offset(newrep->match);
+                nextoffset = Rre.offset(newrep->match);
                 if (nextoffset > srcoff) {
                     memcpy(dstpos, data + srcoff, nextoffset - srcoff);
                     dstpos += nextoffset - srcoff;
@@ -526,7 +538,7 @@ bool DataBuffer::contentRegExp(int filtergroup)
                 replen = newrep->replacement.length();
                 memcpy(dstpos, newrep->replacement.toCharArray(), replen);
                 dstpos += replen;
-                srcoff += re->length(newrep->match);
+                srcoff += Rre.length(newrep->match);
                 delete newrep;
                 matchqueue.pop();
             }
