@@ -6,10 +6,12 @@
 #define __HPP_DATABUFFER
 
 #include <exception>
+#include <memory>
 #include <string.h>
 #include "Socket.hpp"
 #include "String.hpp"
 #include "FDFuncs.hpp"
+#include "FOptionContainer.hpp"
 
 class DMPlugin;
 
@@ -46,11 +48,12 @@ class DataBuffer
     // gives true if it pauses due to too much data
     bool in(Socket *sock, Socket *peersock, class HTTPHeader *requestheader, class HTTPHeader *docheader, bool runav, int *headersent);
     // send body to client
-    void out(Socket *sock) throw(std::exception);
+    bool out(Socket *sock); //throw(std::exception);
 
     void setTimeout(int t)
     {
         timeout = t;
+        stimeout = t * 1000;
     };
     void setDecompress(String d)
     {
@@ -61,7 +64,7 @@ class DataBuffer
     void swapbacktocompressed();
 
     // content regexp search and replace
-    bool contentRegExp(int filtergroup);
+    bool contentRegExp(FOptionContainer* &foc);
 
     // create a temp file and return its FD	- NOT a simple accessor function
     int getTempFileFD();
@@ -80,6 +83,7 @@ class DataBuffer
 #endif
 
     int timeout;
+    int stimeout;
     off_t bytesalreadysent;
     bool preservetemp;
 
