@@ -12,6 +12,7 @@
 #include "DownloadManager.hpp"
 #include "ConfigVar.hpp"
 #include "OptionContainer.hpp"
+#include "ConnectionHandler.hpp"
 #include "RegExp.hpp"
 
 #include <iostream>
@@ -80,19 +81,20 @@ int DMPlugin::init(void *args)
 }
 
 // default method for sending the client a download link
-void DMPlugin::sendLink(Socket &peersock, String &linkurl, String &prettyurl)
+bool DMPlugin::sendLink(Socket &peersock, String &linkurl, String &prettyurl)
 {
     // 1220 "<p>Scan complete.</p><p>Click here to download: "
     String message(o.language_list.getTranslation(1220));
     message += "<a href=\"" + linkurl + "\">" + prettyurl + "</a></p></body></html>\n";
-    peersock.writeString(message.toCharArray());
+    return peersock.writeString(message.toCharArray());
 }
 
 // default method for deciding whether we will handle a request
 bool DMPlugin::willHandle(HTTPHeader *requestheader, HTTPHeader *docheader)
 {
     // match user agent first (quick)
-    if (!(alwaysmatchua || ua_match.match(requestheader->userAgent().toCharArray())))
+    RegResult Rre;
+    if (!(alwaysmatchua || ua_match.match(requestheader->userAgent().toCharArray(),Rre)))
         return false;
 
     // then check standard lists (mimetypes & extensions)
@@ -105,7 +107,8 @@ bool DMPlugin::willHandle(HTTPHeader *requestheader, HTTPHeader *docheader)
 #ifdef DGDEBUG
         std::cout << "mimetype: " << mimetype << std::endl;
 #endif
-        if (mimetypelist.findInList(mimetype.toCharArray()) == NULL) {
+        String lc;
+        if (mimetypelist.findInList(mimetype.toCharArray(), lc) == NULL) {
             if (!extensionlistenabled)
                 return false;
         } else
@@ -147,7 +150,8 @@ bool DMPlugin::willHandle(HTTPHeader *requestheader, HTTPHeader *docheader)
         std::cout << "extension: " << extension << std::endl;
 #endif
         // check the extension list
-        if (!extension.contains(".") || (extensionlist.findEndsWith(extension.toCharArray()) == NULL))
+        String lc;
+        if (!extension.contains(".") || (extensionlist.findEndsWith(extension.toCharArray(), lc) == NULL))
             return matchedmime;
     }
 
