@@ -23,10 +23,9 @@
 
 std::deque<String> *ipToHostname(const char *ip);
 
-class FOptionContainer
-{
+class FOptionContainer {
 
-    public:
+public:
     int reporting_level;
     int category_threshold;
     bool infection_bypass_errors_only;
@@ -53,7 +52,7 @@ class FOptionContainer
     bool ssl_mitm;
     bool only_mitm_ssl_grey;
     bool mitm_check_cert;
-
+    HTMLTemplate html_template;
 #ifdef ENABLE_EMAIL
     // Email notification patch by J. Gauthier
     bool notifyav;
@@ -75,6 +74,9 @@ class FOptionContainer
     bool force_quick_search;
     int bypass_mode;
     int infection_bypass_mode;
+
+#ifdef NOTDEF
+    // pics now disabled so do not need these
     int pics_rsac_violence;
     int pics_rsac_sex;
     int pics_rsac_nudity;
@@ -154,6 +156,7 @@ class FOptionContainer
     int pics_safenet_language;
     int pics_safenet_gambling;
     int pics_safenet_alcoholtobacco;
+#endif
 
     std::string name;
     std::string magic;
@@ -177,6 +180,7 @@ class FOptionContainer
     unsigned int banned_extension_list;
     unsigned int banned_mimetype_list;
     unsigned int banned_site_list;
+    unsigned int banned_site_list_withbypass;
     unsigned int banned_url_list;
     unsigned int grey_site_list;
     unsigned int grey_url_list;
@@ -223,38 +227,38 @@ class FOptionContainer
     unsigned int no_check_cert_site_list;
 
     unsigned int url_redirect_regexp_list;
-    std::deque<RegExp> url_redirect_regexp_list_comp;
-    std::deque<String> url_redirect_regexp_list_rep;
+    std::deque <RegExp> url_redirect_regexp_list_comp;
+    std::deque <String> url_redirect_regexp_list_rep;
     bool url_redirect_regexp_flag;
     bool allow_empty_host_certs;
 
     // regex match lists
-    std::deque<RegExp> banned_regexpurl_list_comp;
-    std::deque<String> banned_regexpurl_list_source;
+    std::deque <RegExp> banned_regexpurl_list_comp;
+    std::deque <String> banned_regexpurl_list_source;
     std::deque<unsigned int> banned_regexpurl_list_ref;
-    std::deque<RegExp> exception_regexpurl_list_comp;
-    std::deque<String> exception_regexpurl_list_source;
+    std::deque <RegExp> exception_regexpurl_list_comp;
+    std::deque <String> exception_regexpurl_list_source;
     std::deque<unsigned int> exception_regexpurl_list_ref;
-    std::deque<RegExp> banned_regexpheader_list_comp;
-    std::deque<String> banned_regexpheader_list_source;
+    std::deque <RegExp> banned_regexpheader_list_comp;
+    std::deque <String> banned_regexpheader_list_source;
     std::deque<unsigned int> banned_regexpheader_list_ref;
-    std::deque<RegExp> log_regexpurl_list_comp;
-    std::deque<String> log_regexpurl_list_source;
+    std::deque <RegExp> log_regexpurl_list_comp;
+    std::deque <String> log_regexpurl_list_source;
     std::deque<unsigned int> log_regexpurl_list_ref;
 
     // regex search & replace lists
-    std::deque<RegExp> content_regexp_list_comp;
-    std::deque<String> content_regexp_list_rep;
-    std::deque<RegExp> url_regexp_list_comp;
-    std::deque<String> url_regexp_list_rep;
-    std::deque<RegExp> sslsite_regexp_list_comp;
-    std::deque<String> sslsite_regexp_list_rep;
-    std::deque<RegExp> header_regexp_list_comp;
-    std::deque<String> header_regexp_list_rep;
-    std::deque<RegExp> search_regexp_list_comp;
-    std::deque<String> search_regexp_list_rep;
-    std::deque<RegExp> addheader_regexp_list_comp;
-    std::deque<String> addheader_regexp_list_rep;
+    std::deque <RegExp> content_regexp_list_comp;
+    std::deque <String> content_regexp_list_rep;
+    std::deque <RegExp> url_regexp_list_comp;
+    std::deque <String> url_regexp_list_rep;
+    std::deque <RegExp> sslsite_regexp_list_comp;
+    std::deque <String> sslsite_regexp_list_rep;
+    std::deque <RegExp> header_regexp_list_comp;
+    std::deque <String> header_regexp_list_rep;
+    std::deque <RegExp> search_regexp_list_comp;
+    std::deque <String> search_regexp_list_rep;
+    std::deque <RegExp> addheader_regexp_list_comp;
+    std::deque <String> addheader_regexp_list_rep;
 
     // precompiled reg exps for speed
     RegExp pics1;
@@ -276,127 +280,55 @@ class FOptionContainer
     //std::deque<unsigned int> searchengine_regexp_list_ref;
     //bool extractSearchTerms(String url, String &terms);
 
-    FOptionContainer()
-        : block_downloads(false), searchterm_flag(false), banned_page(NULL), ssl_mitm(false), only_mitm_ssl_grey(false), no_check_cert_site_flag(false), ssl_check_cert(false), mitm_check_cert(true), referer_exception_site_flag(false), referer_exception_url_flag(false), embeded_referer_site_flag(false), embeded_referer_url_flag(false),
-#ifdef PRT_DNSAUTH
-        auth_exception_site_flag(false)
-        , auth_exception_url_flag(false)
-        ,
-#endif
-        addheader_regexp_flag(false)
-        , banned_search_flag(false)
-        , search_regexp_flag(false)
-        , local_banned_search_flag(false)
-        , banned_search_overide_flag(false)
-        , local_exception_site_flag(false)
-        , local_exception_url_flag(false)
-        , local_banned_site_flag(false)
-        , local_banned_url_flag(false)
-        , local_grey_site_flag(false)
-        , local_grey_url_flag(false)
-        , enable_regex_grey(false)
-        , enable_local_list(false)
-        , enable_ssl_legacy_logic(false)
-        , use_only_local_allow_lists(false)
-        , banned_phrase_flag(false)
-        , exception_site_flag(false)
-        , exception_url_flag(false)
-        , banned_extension_flag(false)
-        , banned_mimetype_flag(false)
-        , banned_site_flag(false)
-        , banned_url_flag(false)
-        , grey_site_flag(false)
-        , grey_url_flag(false)
-        , banned_regexpurl_flag(false)
-        , exception_regexpurl_flag(false)
-        , banned_regexpheader_flag(false)
-        , content_regexp_flag(false)
-        , url_regexp_flag(false)
-        , sslsite_regexp_flag(false)
-        , header_regexp_flag(false)
-        , url_redirect_regexp_flag(false)
-        , exception_extension_flag(false)
-        , exception_mimetype_flag(false)
-        , exception_file_site_flag(false)
-        , exception_file_url_flag(false)
-        , log_site_flag(false)
-        , log_url_flag(false)
-        , log_regexpurl_flag(false)
-        , ssl_denied_rewrite(false)
-        , allow_empty_host_certs(false)
-        , pics_icra_nuditymalegraphic(0)
-        , pics_icra_nudityfemalegraphic(0)
-        , pics_icra_nuditytopless(0)
-        , pics_icra_nuditybottoms(0)
-        , pics_icra_nuditysexualacts(0)
-        , pics_icra_nudityobscuredsexualacts(0)
-        , pics_icra_nuditysexualtouching(0)
-        , pics_icra_nuditykissing(0)
-        , pics_icra_nudityartistic(0)
-        , pics_icra_nudityeducational(0)
-        , pics_icra_nuditymedical(0)
-        , pics_icra_drugstobacco(0)
-        , pics_icra_drugsalcohol(0)
-        , pics_icra_drugsuse(0)
-        , pics_icra_gambling(0)
-        , pics_icra_weaponuse(0)
-        , pics_icra_intolerance(0)
-        , pics_icra_badexample(0)
-        , pics_icra_pgmaterial(0)
-        , pics_icra_violencerape(0)
-        , pics_icra_violencetohumans(0)
-        , pics_icra_violencetoanimals(0)
-        , pics_icra_violencetofantasy(0)
-        , pics_icra_violencekillinghumans(0)
-        , pics_icra_violencekillinganimals(0)
-        , pics_icra_violencekillingfantasy(0){};
+    FOptionContainer();
     ~FOptionContainer();
     bool read(const char *filename);
     void reset();
     void resetJustListData();
 
     bool isOurWebserver(String url);
-    char *inAddheaderList(String &words, unsigned int list);
-    char *inBannedSearchList(String words);
-    char *inSearchList(String &words, unsigned int list);
-    char *inLocalBannedSearchList(String words);
+    char *inAddheaderList(String &words, unsigned int list,String &lastcategory);
+    char *inBannedSearchList(String words,String &lastcategory);
+    char *inSearchList(String &words, unsigned int list,String &lastcategory);
+    char *inLocalBannedSearchList(String words,String &lastcategory);
     bool inBannedSearchOverideList(String words);
-    char *inLocalBannedSiteList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
-    char *inLocalBannedSSLSiteList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
-    char *inLocalBannedURLList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
+    char *inLocalBannedSiteList(String url, bool doblanket , bool ip , bool ssl ,String &lastcategory);
+    char *inLocalBannedSSLSiteList(String url, bool doblanket , bool ip , bool ssl ,String &lastcategory);
+    char *inLocalBannedURLList(String url, bool doblanket , bool ip , bool ssl ,String &lastcategory);
     bool inLocalGreySiteList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
     bool inLocalGreySSLSiteList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
     bool inLocalGreyURLList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
-    bool inLocalExceptionSiteList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
-    bool inLocalExceptionURLList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
-    char *inBannedSSLSiteList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
+    bool inLocalExceptionSiteList(String url, bool doblanket , bool ip , bool ssl ,String &lastcategory);
+    bool inLocalExceptionURLList(String url, bool doblanket , bool ip , bool ssl ,String &lastcategory);
+    char *inBannedSSLSiteList(String url, bool doblanket , bool ip , bool ssl ,String &lastcategory);
     bool inGreySSLSiteList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
-    bool inNoCheckCertSiteList(String url, bool ip = false);
+    bool inNoCheckCertSiteList(String url, bool ip);
 #ifdef PRT_DNSAUTH
     bool inAuthExceptionSiteList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
     bool inAuthExceptionURLList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
 #endif
     bool inRefererExceptionLists(String url);
     bool inEmbededRefererLists(String url);
-    char *inBannedSiteList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
-    char *inBannedURLList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
+    char *inBannedSiteList(String url, bool doblanket , bool ip , bool ssl ,String &lastcategory);
+    char *inBannedSiteListwithbypass(String url, bool doblanket , bool ip , bool ssl ,String &lastcategory);
+    char *inBannedURLList(String url, bool doblanket , bool ip , bool ssl ,String &lastcategory);
     bool inGreySiteList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
     bool inGreyURLList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
-    bool inExceptionSiteList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
-    bool inExceptionURLList(String url, bool doblanket = false, bool ip = false, bool ssl = false);
+    bool inExceptionSiteList(String url, bool doblanket , bool ip , bool ssl ,String &lastcategory);
+    bool inExceptionURLList(String url, bool doblanket , bool ip , bool ssl ,String &lastcategory);
     bool inExceptionFileSiteList(String url);
-    int inBannedRegExpURLList(String url);
-    int inExceptionRegExpURLList(String url);
-    int inBannedRegExpHeaderList(std::deque<String> &header);
+    int inBannedRegExpURLList(String url,String &lastcategory);
+    int inExceptionRegExpURLList(String url,String &lastcategory);
+    int inBannedRegExpHeaderList(std::deque<String> &header,String &lastcategory);
     char *inExtensionList(unsigned int list, String url);
     bool isIPHostname(String url);
     bool addheader_regexp_flag; // public as used by HTTPHeader.cpp
-    bool search_regexp_flag;    // public as used by HTTPHeader.cpp
-    bool sslsite_regexp_flag;   // public as used by HTTPHeader.cpp
+    bool search_regexp_flag; // public as used by HTTPHeader.cpp
+    bool sslsite_regexp_flag;// public as used by HTTPHeader.cpp
 
     // log-only lists - return category
-    const char *inLogURLList(String url);
-    const char *inLogSiteList(String url);
+    const char *inLogURLList(String url, String &lastcat);
+    const char *inLogSiteList(String url, String &lastcat);
     const char *inLogRegExpURLList(String url);
 
     // get HTML template for this group
@@ -413,6 +345,7 @@ class FOptionContainer
     bool banned_extension_flag;
     bool banned_mimetype_flag;
     bool banned_site_flag;
+    bool banned_site_withbypass_flag;
     bool banned_url_flag;
     bool grey_site_flag;
     bool grey_url_flag;
@@ -474,13 +407,13 @@ class FOptionContainer
 
     int findoptionI(const char *option);
     std::string findoptionS(const char *option);
-    bool realitycheck(long int l, long int minl, long int maxl, const char *emessage);
-    int inRegExpURLList(String &url, std::deque<RegExp> &list_comp, std::deque<unsigned int> &list_ref, unsigned int list);
+    bool realitycheck(int l, int minl, int maxl, const char *emessage);
+    int inRegExpURLList(String &url, std::deque<RegExp> &list_comp, std::deque<unsigned int> &list_ref, unsigned int list, String &lastcategory);
 
-    char *inURLList(String &url, unsigned int list, bool doblanket = false, bool ip = false, bool ssl = false);
-    char *inSiteList(String &url, unsigned int list, bool doblanket = false, bool ip = false, bool ssl = false);
+    char *inURLList(String &url, unsigned int list, bool doblanket , bool ip , bool ssl , String &lastcategory);
+    char *inSiteList(String &url, unsigned int list, bool doblanket , bool ip , bool ssl , String &lastcategory);
 
-    char *testBlanketBlock(unsigned int list, bool ip, bool ssl);
+    char *testBlanketBlock(unsigned int list, bool ip, bool ssl, String &lastcategory);
 };
 
 #endif
