@@ -11,6 +11,9 @@
 #define __DGHEADER_SENDFIRSTLINE 1
 #define __DGHEADER_SENDREST 2
 
+#define  __HEADER_REQUEST      1
+#define  __HEADER_RESPONSE   2
+
 // INCLUDES
 
 #include <deque>
@@ -19,6 +22,7 @@
 //#include "DataBuffer.hpp"
 #include "Socket.hpp"
 #include "RegExp.hpp"
+#include "FOptionContainer.hpp"
 
 // DECLARATIONS
 
@@ -28,6 +32,7 @@ class HTTPHeader
     std::deque<String> header;
     //DataBuffer postdata;
     unsigned int port;
+    bool is_response;
 
     // reset header object for future use
     void reset();
@@ -35,14 +40,15 @@ class HTTPHeader
     // network communication funcs
 
     void setTimeout(int t);
-    void in(Socket *sock, bool allowpersistent = false, bool honour_reloadconfig = false);
+    bool in(Socket *sock, bool allowpersistent = false, bool honour_reloadconfig = false);
 
     // send headers out over the given socket
     // "reconnect" flag gives permission to reconnect to the socket on write error
     // - this allows us to re-open the proxy connection on pconns if squid's end has
     // timed out but the client's end hasn't. not much use with NTLM, since squid
     // will throw a 407 and restart negotiation, but works well with basic & others.
-    void out(Socket *peersock, Socket *sock, int sendflag, bool reconnect = false) throw(std::exception);
+    //void out(Socket *peersock, Socket *sock, int sendflag, bool reconnect = false) throw(std::exception);
+    bool out(Socket *peersock, Socket *sock, int sendflag, bool reconnect = false);
 
     // discard remainder of POST data
     // amount to discard can be passed in, or will default to contentLength()
@@ -58,24 +64,28 @@ class HTTPHeader
     String getContentType();
     String getMIMEBoundary();
     // check received content type against given content type
-    bool isContentType(const String &t,int filtergroup);
+    //bool isContentType(const String &t,int filtergroup);
+    bool isContentType(const String &t,FOptionContainer* &foc);
     // check HTTP message code to see if it's an auth required message
     bool authRequired();
     // Content-Disposition
     String disposition();
     String userAgent();
-    String logHeader();
     // grab contents of X-Forwarded-For
     std::string getXForwardedForIP();
     // check HTTP message code to see if it's a redirect
     bool isRedirection();
     // see if content-type is something other than "identity"
     bool isCompressed();
-    bool isHeaderAdded(int filtergroup);
+    //bool isHeaderAdded(int filtergroup);
+    bool isHeaderAdded(FOptionContainer* &foc);
     bool addheaderchecked;
     bool isheaderadded;
+    String *plogheadervalue;
+    String *pheaderident;
+    std::string getAuthHeader();
     // see if search usl and set searchwords
-    bool isSearch(int filtergroup);
+    bool isSearch(FOptionContainer* &foc);
     String searchwords();
     String searchterms();
     bool searchchecked;
@@ -83,9 +93,11 @@ class HTTPHeader
     // grab the contents of Proxy-Authorization header
     // returns base64-decoding of the chunk of data after the auth type string
     std::string getAuthData();
-    std::string getAuthHeader();
     // grab raw contents of Proxy-Authorization header, without b64 decode
     std::string getRawAuthData();
+    // Debug show header
+    void dbshowheader(String *url, const char *clientip);
+    void dbshowheader(bool outgoing);
     // check whether a connection is persistent
     bool isPersistent()
     {
@@ -119,11 +131,11 @@ class HTTPHeader
     void removeEncoding(int newlen);
     void setContentLength(int newlen);
     // regexp search and replace
-    bool urlRegExp(int filtergroup);
-    bool sslsiteRegExp(int filtergroup);
-    bool urlRedirectRegExp(int filtergroup);
-    bool DenySSL(int filtergroup);
-    bool headerRegExp(int filtergroup);
+    bool urlRegExp(FOptionContainer* &foc);
+    bool sslsiteRegExp(FOptionContainer* &foc);
+    bool urlRedirectRegExp(FOptionContainer* &foc);
+    bool DenySSL(FOptionContainer* &foc);
+    bool headerRegExp(FOptionContainer* &foc);
     // make a connection persistent - or not
     void makePersistent(bool persist = true);
     // make the request look as if its coming from the origin server
@@ -160,10 +172,17 @@ class HTTPHeader
     String getReferer();
 
     HTTPHeader()
-        : port(0), timeout(120), contentlength(0), postdata(NULL), dirty(true)
+        : port(0), timeout(120000), contentlength(0), postdata(NULL), dirty(true), is_response(false)
     {
         reset();
     };
+    HTTPHeader(int type)
+            : port(0), timeout(120000), contentlength(0), postdata(NULL), dirty(true), is_response(false)
+    {
+        reset();
+        if (type == __HEADER_RESPONSE)
+            is_response = true;
+    };
     ~HTTPHeader()
     {
         delete postdata;
@@ -183,20 +202,21 @@ class HTTPHeader
     String *pproxyauthenticate;
     String *pcontentdisposition;
     String *puseragent;
-    String *plogheadervalue;
-    String *pheaderident;
     String *pxforwardedfor;
     String *pcontentencoding;
     String *pproxyconnection;
     String *pkeepalive;
+
     // cached result of getUrl()
     std::string cachedurl;
     // used to record if it is a header within a MITM
     bool mitm;
 
     String redirect;
-    std::string searchwds;
-    std::string searchtms;
+    String searchwds;
+    //std::string searchwds;
+    //std::string searchtms;
+    String searchtms;
     bool issearch;
 
     // cached result of contentLength()
