@@ -14,7 +14,6 @@
 #include "ListContainer.hpp"
 #include "OptionContainer.hpp"
 #include "RegExp.hpp"
-#include "SysV.hpp"
 #include <cstdlib>
 #include <cstdio>
 #include <ctime>
@@ -337,17 +336,40 @@ bool ListContainer::ifsreadItemList(std::ifstream *input, int len, bool checkend
 {
     RegExp re;
     re.comp("^.*\\:[0-9]+\\/.*");
+    RegResult Rre;
 #ifdef DGDEBUG
     if (filters != 32)
         std::cout << "Converting to lowercase" << std::endl;
 #endif
     increaseMemoryBy(len + 2); // Allocate some memory to hold file
     String temp, inc, hostname, url;
-    char linebuffer[2048];
-    while (input->getline(linebuffer, sizeof(linebuffer))) {
+    //char linebuffer[2048];
+    char linebuffer[20000];    // increased to allow checking of line length
+    // NOTE: should check for errors instead but input->fail() always seems to
+    // be true after first failure and input->clear() does not appear to work
+    //  - for future investigation
+
+   // bool skip = false;
+    while (!input->eof()) {
+        input->getline(linebuffer, sizeof(linebuffer));
+     //   if (input->fail()) {
+     //       skip = true;
+      //      syslog(LOG_ERR, "Line input failure" );
+       //     input->clear();
+        //    break;
+        //}
+        //if (skip) {
+          //  skip = false;
+           // continue;
+        //}
         temp = linebuffer;
         if (temp.length() < 2)
             continue; // its jibberish
+        if (temp.length() > 2048) {
+            temp.limitLength(100);
+            syslog(LOG_ERR, "Line too long in list file - ignored %s....", temp.toCharArray() );
+            continue;
+        }
 
         //item lists (URLs, domains) can be both categorised and time-limited
         if (linebuffer[0] == '#') {
@@ -421,7 +443,7 @@ bool ListContainer::ifsreadItemList(std::ifstream *input, int len, bool checkend
         }
         if (filters == 1) { // remove port addresses
             if (temp.before("/").contains(":")) { // quicker than full regexp
-                if (re.match(temp.toCharArray())) {
+                if (re.match(temp.toCharArray(),Rre)) {
                     hostname = temp.before(":");
                     url = temp.after("/");
                     temp = hostname + "/" + url;
@@ -470,23 +492,11 @@ bool ListContainer::readItemList(const char *filename, bool startswith, int filt
     sourcefilters = filters;
     std::string linebuffer;
 #ifdef DGDEBUG
-    std::cout << filename << std::endl;
+    std::cout << "Read filename: " << filename << std::endl;
 #endif
     // see if cached .process file is available and up to date,
-    // if prefercachedlists has been turned on (SG) (CN)
+    // or prefercachedlists has been turned on (SG)
     struct stat s;
-    if ((o.prefer_cached_lists && stat(std::string(linebuffer).append(".processed").c_str(), &s) == 0)) {
-        if (isCacheFileNewer(filename)) {
-            linebuffer = filename;
-            linebuffer += ".processed";
-            // read cached
-            if (!readProcessedItemList(linebuffer.c_str(), startswith, filters))
-                return false;
-            filedate = getFileDate(linebuffer.c_str());
-            issorted = true; // don't bother sorting cached file
-            return true;
-        }
-    }
     filedate = getFileDate(filename);
     size_t len = 0;
     try {
@@ -536,6 +546,7 @@ bool ListContainer::readStdinItemList(bool startswith, int filters, const char *
     std::string linebuffer;
     RegExp re;
     re.comp("^.*\\:[0-9]+\\/.*");
+    RegResult Rre;
     size_t len = 0;
     increaseMemoryBy(2048); // Allocate some memory to hold list
     if (!std::cin.good()) {
@@ -587,7 +598,7 @@ bool ListContainer::readStdinItemList(bool startswith, int filters, const char *
         }
         if (filters == 1) { // remove port addresses
             if (temp.before("/").contains(":")) { // quicker than full regexp
-                if (re.match(temp.toCharArray())) {
+                if (re.match(temp.toCharArray(),Rre)) {
                     hostname = temp.before(":");
                     url = temp.after("/");
                     temp = hostname + "/" + url;
@@ -625,16 +636,16 @@ bool ListContainer::readAnotherItemList(const char *filename, bool startswith, i
 }
 
 // for item lists - is this item in the list?
-bool ListContainer::inList(const char *string)
+bool ListContainer::inList(const char *string, String &lastcategory)
 {
-    if (findInList(string) != NULL) {
+    if (findInList(string, lastcategory) != NULL) {
         return true;
     }
     return false;
 }
 
 // for item lists - is an item in the list that ends with this string?
-bool ListContainer::inListEndsWith(const char *string)
+bool ListContainer::inListEndsWith(const char *string, String &lastcategory)
 {
     if (isNow()) {
         if (items > 0) {
@@ -645,9 +656,9 @@ bool ListContainer::inListEndsWith(const char *string)
         }
         bool rc;
         for (unsigned int i = 0; i < morelists.size(); i++) {
-            rc = (*o.lm.l[morelists[i]]).inListEndsWith(string);
+            rc = (*o.lm.l[morelists[i]]).inListEndsWith(string, lastcategory);
             if (rc) {
-                lastcategory = (*o.lm.l[morelists[i]]).lastcategory;
+            //    lastcategory = (*o.lm.l[morelists[i]]).lastcategory;
                 return true;
             }
         }
@@ -656,7 +667,7 @@ bool ListContainer::inListEndsWith(const char *string)
 }
 
 // for item lists - is an item in the list that starts with this string?
-bool ListContainer::inListStartsWith(const char *string)
+bool ListContainer::inListStartsWith(const char *string, String &lastcategory)
 {
     if (isNow()) {
         if (items > 0) {
@@ -667,9 +678,9 @@ bool ListContainer::inListStartsWith(const char *string)
         }
         bool rc;
         for (unsigned int i = 0; i < morelists.size(); i++) {
-            rc = (*o.lm.l[morelists[i]]).inListStartsWith(string);
+            rc = (*o.lm.l[morelists[i]]).inListStartsWith(string, lastcategory);
             if (rc) {
-                lastcategory = (*o.lm.l[morelists[i]]).lastcategory;
+                //lastcategory = (*o.lm.l[morelists[i]]).lastcategory;
                 return true;
             }
         }
@@ -678,7 +689,7 @@ bool ListContainer::inListStartsWith(const char *string)
 }
 
 // find pointer to the part of the data array containing this string
-char *ListContainer::findInList(const char *string)
+char *ListContainer::findInList(const char *string, String &lastcategory)
 {
     if (isNow()) {
         if (items > 0) {
@@ -695,9 +706,9 @@ char *ListContainer::findInList(const char *string)
         }
         char *rc;
         for (unsigned int i = 0; i < morelists.size(); i++) {
-            rc = (*o.lm.l[morelists[i]]).findInList(string);
+            rc = (*o.lm.l[morelists[i]]).findInList(string, lastcategory);
             if (rc != NULL) {
-                lastcategory = (*o.lm.l[morelists[i]]).lastcategory;
+                //lastcategory = (*o.lm.l[morelists[i]]).lastcategory;
                 return rc;
             }
         }
@@ -706,7 +717,7 @@ char *ListContainer::findInList(const char *string)
 }
 
 // find an item in the list which starts with this
-char *ListContainer::findStartsWith(const char *string)
+char *ListContainer::findStartsWith(const char *string, String &lastcategory)
 {
     if (isNow()) {
         if (items > 0) {
@@ -718,9 +729,9 @@ char *ListContainer::findStartsWith(const char *string)
         }
         char *rc;
         for (unsigned int i = 0; i < morelists.size(); i++) {
-            rc = (*o.lm.l[morelists[i]]).findStartsWith(string);
+            rc = (*o.lm.l[morelists[i]]).findStartsWith(string, lastcategory);
             if (rc != NULL) {
-                lastcategory = (*o.lm.l[morelists[i]]).lastcategory;
+                //lastcategory = (*o.lm.l[morelists[i]]).lastcategory;
                 return rc;
             }
         }
@@ -728,7 +739,7 @@ char *ListContainer::findStartsWith(const char *string)
     return NULL;
 }
 
-char *ListContainer::findStartsWithPartial(const char *string)
+char *ListContainer::findStartsWithPartial(const char *string, String &lastcategory)
 {
     if (isNow()) {
         if (items > 0) {
@@ -745,9 +756,9 @@ char *ListContainer::findStartsWithPartial(const char *string)
         }
         char *rc;
         for (unsigned int i = 0; i < morelists.size(); i++) {
-            rc = (*o.lm.l[morelists[i]]).findStartsWithPartial(string);
+            rc = (*o.lm.l[morelists[i]]).findStartsWithPartial(string, lastcategory);
             if (rc != NULL) {
-                lastcategory = (*o.lm.l[morelists[i]]).lastcategory;
+                //lastcategory = (*o.lm.l[morelists[i]]).lastcategory;
                 return rc;
             }
         }
@@ -755,7 +766,7 @@ char *ListContainer::findStartsWithPartial(const char *string)
     return NULL;
 }
 
-char *ListContainer::findEndsWith(const char *string)
+char *ListContainer::findEndsWith(const char *string, String &lastcategory)
 {
     if (isNow()) {
         if (items > 0) {
@@ -767,9 +778,9 @@ char *ListContainer::findEndsWith(const char *string)
         }
         char *rc;
         for (unsigned int i = 0; i < morelists.size(); i++) {
-            rc = (*o.lm.l[morelists[i]]).findEndsWith(string);
+            rc = (*o.lm.l[morelists[i]]).findEndsWith(string, lastcategory);
             if (rc != NULL) {
-                lastcategory = (*o.lm.l[morelists[i]]).lastcategory;
+                //lastcategory = (*o.lm.l[morelists[i]]).lastcategory;
                 return rc;
             }
         }
@@ -872,60 +883,6 @@ void ListContainer::doSort(const bool startsWith)
     return;
 }
 
-bool ListContainer::createCacheFile()
-{
-    unsigned int i;
-    for (i = 0; i < morelists.size(); i++) {
-        (*o.lm.l[morelists[i]]).createCacheFile();
-    }
-    if (isCacheFileNewer(sourcefile.toCharArray())) { // only do if it needs updating
-        return true;
-    }
-    if (items < 1000) { // There is little to gain when there are so few
-        return true;
-    }
-    String f(sourcefile);
-    f += ".processed";
-#ifdef DGDEBUG
-    std::cout << "creating processed file:" << f << std::endl;
-#endif
-    std::ofstream listfile(f.toCharArray(), std::ios::out);
-    if (listfile.fail()) {
-        if (!is_daemonised) {
-            std::cerr << "Error creating cache file. Do you have write access to this area: \"" << f << "\"?" << std::endl;
-        }
-        syslog(LOG_ERR, "Error creating cache file. Do you have write access to this area: \"%s\"?", f.toCharArray());
-        return false;
-    }
-
-    for (i = 0; i < morelists.size(); i++) {
-        f = ".Include<";
-        f += (*o.lm.l[morelists[i]]).sourcefile;
-        f += ">\n";
-        listfile.write(f.toCharArray(), f.length());
-    }
-    if (listtimelimit.timetag.length() > 10) { // no point in writing corrupt short one
-        f = listtimelimit.timetag;
-        f += "\n";
-        listfile.write(f.toCharArray(), f.length());
-    }
-    if (category.length() > 2) { // no point in writing corrupt short one
-        // lengthened from 4 to allow "ADs" category to be preserved in processed files
-        f = "#listcategory:\"";
-        f += category;
-        f += "\"\n";
-        listfile.write(f.toCharArray(), f.length());
-    }
-
-    char *offset;
-    for (i = 0; i < (unsigned)items; i++) { // write the entries in order
-        offset = data + list[i];
-        listfile.write(offset, strlen(offset));
-        listfile.put('\n'); // newline per entry
-    }
-    listfile.close();
-    return true;
-}
 
 bool ListContainer::makeGraph(bool fqs)
 {
@@ -1697,28 +1654,6 @@ int ListContainer::greaterThanEW(const char *a, const char *b)
     return 0; // both equal
 }
 
-bool ListContainer::isCacheFileNewer(const char *filename)
-{
-    size_t len = 0;
-    try {
-        len = getFileLength(filename);
-    } catch (std::runtime_error &e) {
-        if (!is_daemonised) {
-            std::cerr << "Error reading file " << filename << ": " << e.what() << std::endl;
-        }
-        syslog(LOG_ERR, "Error reading file %s: %s", filename, e.what());
-        return false;
-    }
-    int bannedlistdate = getFileDate(filename);
-    std::string linebuffer(filename);
-    linebuffer += ".processed";
-    int cachedate = getFileDate(linebuffer.c_str());
-    if (cachedate < bannedlistdate) {
-        return false; // cache file is older than list file
-    }
-    return true;
-}
-
 void ListContainer::increaseMemoryBy(size_t bytes)
 {
     if (data_memory > 0) {
@@ -1740,42 +1675,54 @@ size_t getFileLength(const char *filename)
         throw std::runtime_error(strerror(errno));
     return status.st_size;
 }
-
 time_t getFileDate(const char *filename)
 {
-    std::string compare = filename;
     struct stat status;
     int rc = stat(filename, &status);
-    if (rc != 0) {
-	// Check only permissions for cache file
-        if (errno == ENOENT && !compare.find(".processed")){
-            #ifdef DGDEBUG
-            std::cout << "Cannot stat file m_time for " << filename << ". stat() returned errno ENOENT." << std::endl;
-            #endif
-            syslog(LOG_ERR, "Error reading %s. Check directory and file permissions. They should be 640 and 750: %s", filename, strerror(errno));
+    if (strlen(filename) < 1)
             return 0;
-        } else if (errno == ENOENT && compare.find(".processed")){
-	    return 0;
-	}
-        // If there are permission problems, just reload the file (CN)
-        if (errno == EACCES) {
-            #ifdef DGDEBUG
-            std::cout << "Cannot stat file m_time for " << filename << ". stat() returned errno EACCES." << std::endl;
-            #endif
+	
+    if (rc != 0) {
+        if (errno == ENOENT) {
+#ifdef DGDEBUG
+            std::cout << "Cannot stat file m_time for " << filename << ". stat() returned errno ENOENT." << std::endl;
+#endif
             syslog(LOG_ERR, "Error reading %s. Check directory and file permissions. They should be 640 and 750: %s", filename, strerror(errno));
             return 0;
         }
+        // If there are permission problems, just reload the file (CN)
+        if (errno == EACCES) {
+#ifdef DGDEBUG
+            std::cout << "Cannot stat file m_time for " << filename << ". stat() returned errno EACCES." << std::endl;
+#endif
+            syslog(LOG_ERR, "Error reading %s. Check directory and file permissions and ownership. They should be 640 and 750 and readable by the e2guardian user: %s", filename, strerror(errno));
+            return 0;
+        }
         else {
-            // We need to die in this case since continuing will cause the process to segfault (CN)
             if (!is_daemonised) {
-                std::cerr << "Error reading " << filename << "Check directory and file permissions. They should be 640 and 750: " << strerror(errno) << std::endl;
+                std::cerr << "Error reading " << filename << "Check directory and file permissions and ownership. They should be 750 and 640 and readable by the e2guardian user: " << strerror(errno) << std::endl;
             }
-            syslog(LOG_ERR, "Error reading %s. Check directory and file permissions. They should be 640 and 750: %s", filename, strerror(errno));
-            return sysv_kill(o.pid_filename);
+            syslog(LOG_ERR, "Error reading %s. Check directory and file permissions and ownership. They should be 750 and 640 and readable by the e2guardian user: %s", filename, strerror(errno));
+            return 0;
+            //return sysv_kill(o.pid_filename);
         }
     }
     return status.st_mtime;
 }
+#ifdef NODEF
+time_t getFileDate(const char *filename)
+{
+    struct stat status;
+    int rc = stat(filename, &status);
+    if (rc != 0) {
+        if (errno == ENOENT)
+            return 0;
+        else
+            throw std::runtime_error(strerror(errno));
+    }
+    return status.st_mtime;
+}
+#endif
 
 bool ListContainer::upToDate()
 {
@@ -1783,19 +1730,6 @@ bool ListContainer::upToDate()
         return false;
     }
 
-    // No need to check for cached files if prefercachedlists has not been turned on
-    // There may be a slight performance improvement by avoiding unnecessary checks (CN)
-    if (o.prefer_cached_lists) {
-        #ifdef DGDEBUG
-        std::cout << "It appears that prefercachedlists is turned on: verifying freshness." << std::endl;
-        #endif
-        String cachefile(sourcefile);
-        cachefile += ".processed";
-        if (getFileDate(cachefile.toCharArray()) > filedate){
-            return false;
-        }
-    }
-
     for (unsigned int i = 0; i < morelists.size(); i++) {
         if (!(*o.lm.l[morelists[i]]).upToDate()) {
             return false;
