@@ -7,6 +7,7 @@
 #ifdef HAVE_CONFIG_H
 #include "dgconfig.h"
 #endif
+#include "LOptionContainer.hpp"
 #include "OptionContainer.hpp"
 #include "RegExp.hpp"
 #include "ConfigVar.hpp"
@@ -23,16 +24,21 @@
 
 extern bool is_daemonised;
 
+
+
 ListContainer total_block_site_list;
 ListContainer total_block_url_list;
 
 // IMPLEMENTATION
 
 OptionContainer::OptionContainer()
-    : use_filter_groups_list(false), auth_needs_proxy_query(false), prefer_cached_lists(false), no_daemon(false), no_logger(false), log_syslog(false), anonymise_logs(false), log_ad_blocks(false), log_timestamp(false), log_user_agent(false), soft_restart(false), delete_downloaded_temp_files(false), max_logitem_length(2000), max_content_filter_size(0), max_content_ramcache_scan_size(0), max_content_filecache_scan_size(0), scan_clean_cache(0), content_scan_exceptions(0), initial_trickle_delay(0), trickle_delay(0), content_scanner_timeout(0), reporting_level(0), weighted_phrase_mode(0), numfg(0), dstat_log_flag(false), dstat_interval(300), fg(NULL)
+    : use_filter_groups_list(false), stats_human_readable(false), auth_requires_user_and_group(false), use_group_names_list(false), auth_needs_proxy_query(false), prefer_cached_lists(false), no_daemon(false), no_logger(false), log_syslog(false), anonymise_logs(false), log_ad_blocks(false), log_timestamp(false), log_user_agent(false), soft_restart(false), delete_downloaded_temp_files(false), max_logitem_length(2000), max_content_filter_size(0), max_content_ramcache_scan_size(0), max_content_filecache_scan_size(0), scan_clean_cache(0), content_scan_exceptions(0), initial_trickle_delay(0), trickle_delay(0), content_scanner_timeout(0), reporting_level(0), weighted_phrase_mode(0), numfg(0), dstat_log_flag(false), dstat_interval(300), dns_user_logging(false), LC_cnt(0)
 {
+    log_Q = new Queue<std::string>;
+    http_worker_Q = new Queue<Socket*>;
 }
 
+
 OptionContainer::~OptionContainer()
 {
     reset();
@@ -40,14 +46,13 @@ OptionContainer::~OptionContainer()
 
 void OptionContainer::reset()
 {
-    deleteFilterGroups();
+    //deleteFilterGroups();
     deletePlugins(dmplugins);
     deletePlugins(csplugins);
     deletePlugins(authplugins);
-    deleteRooms();
-    exception_ip_list.reset();
-    banned_ip_list.reset();
-    html_template.reset();
+    //deleteRooms();
+    //exception_ip_list.reset();
+    //banned_ip_list.reset();
     language_list.reset();
     conffile.clear();
     if (use_filter_groups_list)
@@ -57,31 +62,6 @@ void OptionContainer::reset()
     auth_map.clear();
 }
 
-void OptionContainer::deleteFilterGroups()
-{
-    for (int i = 0; i < numfg; i++) {
-        if (fg[i] != NULL) {
-#ifdef DGDEBUG
-            std::cout << "In deleteFilterGroups loop" << std::endl;
-#endif
-            delete fg[i]; // delete extra FOptionContainer objects
-            fg[i] = NULL;
-        }
-    }
-    if (numfg > 0) {
-        delete[] fg;
-        numfg = 0;
-    }
-}
-
-void OptionContainer::deleteFilterGroupsJustListData()
-{
-    for (int i = 0; i < numfg; i++) {
-        if (fg[i] != NULL) {
-            fg[i]->resetJustListData();
-        }
-    }
-}
 
 void OptionContainer::deletePlugins(std::deque<Plugin *> &list)
 {
@@ -94,17 +74,18 @@ void OptionContainer::deletePlugins(std::deque<Plugin *> &list)
     list.clear();
 }
 
-bool OptionContainer::read(const char *filename, int type)
+bool OptionContainer::read(std::string& filename, int type)
 {
 	conffilename = filename;
+
 	// all sorts of exceptions could occur reading conf files
 	try {
 		std::string linebuffer;
 		String temp;  // for tempory conversion and storage
-		std::ifstream conffiles(filename, std::ios::in);  // e2guardian.conf
+		std::ifstream conffiles(filename.c_str(), std::ios::in);  // e2guardian.conf
 		if (!conffiles.good()) {
 			if (!is_daemonised) {
-				std::cerr << "error reading: " << filename << std::endl;
+				std::cerr << "error reading: " << filename.c_str() << std::endl;
 			}
 			syslog(LOG_ERR, "%s", "error reading e2guardian.conf");
 			return false;
@@ -125,53 +106,56 @@ bool OptionContainer::read(const char *filename, int type)
 		}
 		conffiles.close();
 
-		if (type == 0 || type == 2) {
+                if (type == 0 || type == 2) {
 
-			if ((ipc_filename = findoptionS("ipcfilename")) == "")
-				ipc_filename = "/tmp/.e2guardianipc";
+                    if ((pid_filename = findoptionS("pidfilename")) == "") {
+                        pid_filename = __PIDDIR;
+                        pid_filename += "/e2guardian.pid";
+                    }
 
-			if ((urlipc_filename = findoptionS("urlipcfilename")) == "")
-				urlipc_filename = "/tmp/.e2guardianurlipc";
+                    if (findoptionS("logsyslog") == "on") {
+                        log_syslog = true;
+                        if ((name_suffix = findoptionS("namesuffix")) == "") {
+                            name_suffix = "";
+                        }
+                    } else 	if ((log_location = findoptionS("loglocation")) == "") {
+                        log_location = __LOGLOCATION;
+                        log_location += "/access.log";
+                        log_syslog = false;
+                    }
 
-			if ((ipipc_filename = findoptionS("ipipcfilename")) == "")
-				ipipc_filename = "/tmp/.e2guardianipipc";
+                    if ((stat_location = findoptionS("statlocation")) == "") {
+                        stat_location = __LOGLOCATION;
+                        stat_location += "/stats";
+                    }
 
-			if ((pid_filename = findoptionS("pidfilename")) == "") {
-				pid_filename = __PIDDIR;
-				pid_filename += "/e2guardian.pid";
-			}
+                    if ((dstat_location = findoptionS("dstatlocation")) == "") {
+                        dstat_log_flag = false;
+                    } else {
+                        dstat_log_flag = true;
+                        dstat_interval = findoptionI("dstatinterval");
+                        if ( dstat_interval  == 0) {
+                            dstat_interval = 300; // 5 mins
+                        }
+                    }
 
-			if (findoptionS("logsyslog") == "on") {
-				log_syslog = true;
-        if ((name_suffix = findoptionS("namesuffix")) == "") {
-          name_suffix = "";
-        }
-			} else 	if ((log_location = findoptionS("loglocation")) == "") {
-				log_location = __LOGLOCATION;
-				log_location += "/access.log";
-				log_syslog = false;
-			}
+                    if (findoptionS("statshumanreadable") == "on") {
+                        stats_human_readable = true;
+                    } else {
+                        stats_human_readable = false;
+                    }
 
-			if ((stat_location = findoptionS("statlocation")) == "") {
-				stat_location = __LOGLOCATION;
-				stat_location += "/stats";
-			}
+                    if ((dns_user_logging_domain = findoptionS("dnsuserloggingdomain")) == "") {
+                        dns_user_logging = false;
+                    } else {
+                        dns_user_logging = true;
+                    }
 
-			if ((dstat_location = findoptionS("dstatlocation")) == "") {
-				dstat_log_flag = false;
-			} else {
-				dstat_log_flag = true;
-				dstat_interval = findoptionI("dstatinterval");
-				if ( dstat_interval  == 0) {
-					dstat_interval = 300; // 5 mins
-				}
-			}
-
-
-			if (type == 0) {
-				return true;
-			}
-		}
+                    log_header_value = findoptionS("logheadervalue");
+                    if (type == 0) {
+                        return true;
+                    }
+                }
 
 		if ((daemon_user_name = findoptionS("daemonuser")) == "") {
 			daemon_user_name = __PROXYUSER;
@@ -209,25 +193,48 @@ bool OptionContainer::read(const char *filename, int type)
 #endif
 
 #ifdef __SSLMITM
-        // TODO: maybe make these more sensible paths?
+        if (findoptionS("enablessl") == "on") {
+               enable_ssl  = true;
+            } else {
+                enable_ssl = false;
+            }
+
+       if(enable_ssl) {
+        bool ret = true;
         ca_certificate_path = findoptionS("cacertificatepath");
         if (ca_certificate_path == "") {
-            //ca_certificate_path = __CONFDIR "/ca.pem";
+		   if (!is_daemonised){
+                    std::cerr << "cacertificatepath is required when ssl is enabled" << std::endl;
+            }
+            syslog(LOG_ERR, "%s", "cacertificatepath is required when ssl is enabled");
+             ret = false;
         }
 
         ca_private_key_path = findoptionS("caprivatekeypath");
         if (ca_private_key_path == "") {
-            //ca_private_key_path = __CONFDIR "/ca.key";
+		   if (!is_daemonised){
+                    std::cerr << "caprivatekeypath is required when ssl is enabled" << std::endl;
+            }
+            syslog(LOG_ERR, "%s", "caprivatekeypath is required when ssl is enabled");
+             ret = false;
         }
 
         cert_private_key_path = findoptionS("certprivatekeypath");
         if (cert_private_key_path == "") {
-            //cert_private_key_path = __CONFDIR "/certs.key";
+		   if (!is_daemonised){
+                    std::cerr << "certprivatekeypath is required when ssl is enabled" << std::endl;
+            }
+            syslog(LOG_ERR, "%s", "certprivatekeypath is required when ssl is enabled");
+             ret = false;
         }
 
         generated_cert_path = findoptionS("generatedcertpath") + "/";
         if (generated_cert_path == "/") {
-            //generated_cert_path = "/etc/ssl/certs/";
+		   if (!is_daemonised){
+                    std::cerr << "generatedcertpath is required when ssl is enabled" << std::endl;
+            }
+            syslog(LOG_ERR, "%s", "generatedcertpath is required when ssl is enabled");
+             ret = false;
         }
 
         time_t gen_cert_start, gen_cert_end;
@@ -244,12 +251,16 @@ bool OptionContainer::read(const char *filename, int type)
         if (set_cipher_list == "")
             set_cipher_list = "HIGH:!ADH:!MD5:!RC4:!SRP:!PSK:!DSS";
 
-        if (ca_certificate_path != "") {
-            ca = new CertificateAuthority(ca_certificate_path.c_str(),
+//        if (ca_certificate_path != "" )
+        if (ret) {
+            	ca = new CertificateAuthority(ca_certificate_path.c_str(),
                 ca_private_key_path.c_str(),
                 cert_private_key_path.c_str(),
                 generated_cert_path.c_str(),
                 gen_cert_start, gen_cert_end);
+        } else {
+                return false;
+            }
         }
 
 #endif
@@ -259,8 +270,13 @@ bool OptionContainer::read(const char *filename, int type)
         mailer = findoptionS("mailer");
 #endif
 
-        // the e2guardian.conf and pics files get amalgamated into one
-        // deque.  They are only seperate files for clarity.
+        max_header_lines = findoptionI("maxheaderlines");
+        if (max_header_lines == 0)
+            max_header_lines= 40;
+        if (!realitycheck(max_header_lines, 10, 250, "maxheaderlines")) {
+            return false;
+        }
+
 
         max_logitem_length = findoptionI("maxlogitemlength");
         // default of unlimited no longer allowed as could cause buffer overflow
@@ -270,62 +286,42 @@ bool OptionContainer::read(const char *filename, int type)
             return false;
         }
 
-        proxy_timeout = findoptionI("proxytimeout");
-        if (!realitycheck(proxy_timeout, 5, 100, "proxytimeout")) {
+        proxy_timeout_sec = findoptionI("proxytimeout");
+        if (!realitycheck(proxy_timeout_sec, 5, 100, "proxytimeout")) {
             return false;
         } // check its a reasonable value
+        proxy_timeout = proxy_timeout_sec * 1000;
 
         proxy_failure_log_interval = findoptionI("proxyfailureloginterval");
         if (proxy_failure_log_interval == 0)
             proxy_failure_log_interval = 600; // 10 mins
-        if (!realitycheck(proxy_failure_log_interval, proxy_timeout, 3600, "proxyfailureloginterval")) {
+        if (!realitycheck(proxy_failure_log_interval, proxy_timeout_sec, 3600, "proxyfailureloginterval")) {
             return false;
         } // check its a reasonable value
 
-        pcon_timeout = findoptionI("pcontimeout");
-        if (!realitycheck(pcon_timeout, 5, 300, "pcontimeout")) {
+        pcon_timeout_sec = findoptionI("pcontimeout");
+        if (!realitycheck(pcon_timeout_sec, 5, 300, "pcontimeout")) {
             return false;
         } // check its a reasonable value
+        pcon_timeout = pcon_timeout_sec * 1000;
 
-        exchange_timeout = findoptionI("proxyexchange");
-        if (!realitycheck(exchange_timeout, 5, 300, "proxyexchange")) {
+        exchange_timeout_sec = findoptionI("proxyexchange");
+        if (!realitycheck(exchange_timeout_sec, 5, 300, "proxyexchange")) {
             return false;
         }
+        exchange_timeout = exchange_timeout_sec  * 1000;
 
-        max_children = findoptionI("maxchildren");
-        if (!realitycheck(max_children, 4, 0, "maxchildren")) {
-            return false;
+        http_workers= findoptionI("httpworkers");
+        if (http_workers == 0) {
+		http_workers = 100;
+		if (!is_daemonised){
+               		std::cerr << " http_workers settings cannot be zero: value set to 100" << std::endl;
+		}
+                syslog(LOG_ERR, "http_workers settings cannot be zero: value set to 100");
+	}
+        if (!realitycheck(http_workers, 20, 20000, "httpworkers")) {
+        return false;
         } // check its a reasonable value
-        min_children = findoptionI("minchildren");
-        if (!realitycheck(min_children, 1, max_children - 1, "minchildren")) {
-            return false;
-        } // check its a reasonable value
-        maxspare_children = findoptionI("maxsparechildren");
-        if (!realitycheck(maxspare_children, min_children, max_children, "maxsparechildren")) {
-           return false;
-        } // check its a reasonable value
-        prefork_children = findoptionI("preforkchildren");
-        if (!realitycheck(prefork_children, 1, max_children, "preforkchildren")) {
-            return false;
-        } // check its a reasonable value
-        minspare_children = findoptionI("minsparechildren");
-        if (!realitycheck(minspare_children, 0, maxspare_children - 1, "minsparechildren")) {
-            return false;
-        } // check its a reasonable value
-        maxage_children = findoptionI("maxagechildren");
-        if (!realitycheck(maxage_children, 1, 0, "maxagechildren")) {
-            return false;
-        } // check its a reasonable value
-
-        gentle_chunk = findoptionI("gentlechunk");
-        if (gentle_chunk > 0) {
-            if (!realitycheck(gentle_chunk, 1, min_children, "gentlechunk")) {
-                return false;
-            } // check its a reasonable value
-        } else {
-            gentle_chunk = prefork_children;
-        }
-        monitor_start = 0;
 
         monitor_helper = findoptionS("monitorhelper");
         if (monitor_helper == "") {
@@ -341,20 +337,6 @@ bool OptionContainer::read(const char *filename, int type)
             monitor_flag_flag = true;
         }
 
-        if (monitor_helper_flag || monitor_flag_flag) {
-            monitor_start = findoptionI("monitorstart");
-            if (!realitycheck(monitor_start, 0, min_children, "monitorstart")) {
-                return false;
-            } // check its a reasonable value
-            if (monitor_start == 0)
-                monitor_start = min_children;
-        }
-
-        max_ips = findoptionI("maxips");
-        if (!realitycheck(max_ips, 0, 0, "maxips")) {
-            return false;
-        }
-
         max_content_filter_size = findoptionI("maxcontentfiltersize");
         if (!realitycheck(max_content_filter_size, 0, 0, "maxcontentfiltersize")) {
             return false;
@@ -382,6 +364,21 @@ bool OptionContainer::read(const char *filename, int type)
             max_content_ramcache_scan_size = max_content_filecache_scan_size;
         }
 
+        weighted_phrase_mode = findoptionI("weightedphrasemode");
+        if (!realitycheck(weighted_phrase_mode, 0, 2, "weightedphrasemode")) {
+            return false;
+        }
+        /*
+		if ((weighted_phrase_mode != 0)) {
+			max_content_filter_size = max_content_ramcache_scan_size;
+			if (max_content_filter_size == 0) {
+				if (!is_daemonised)
+					std::cerr << "maxcontent* settings cannot be zero (to disable phrase filtering, set weightedphrasemode to 0)" << std::endl;
+				syslog(LOG_ERR, "%s", "maxcontent* settings cannot be zero (to disable phrase filtering, set weightedphrasemode to 0)");
+				return false;
+			}
+		}
+*/
         bool contentscanning = findoptionM("contentscanner").size() > 0;
         if (contentscanning) {
 
@@ -409,10 +406,11 @@ bool OptionContainer::read(const char *filename, int type)
                 return false;
             }
 
-            content_scanner_timeout = findoptionI("contentscannertimeout");
-            if (!realitycheck(content_scanner_timeout, 1, 0, "contentscannertimeout")) {
+            content_scanner_timeout_sec = findoptionI("contentscannertimeout");
+            if (!realitycheck(content_scanner_timeout_sec, 1, 0, "contentscannertimeout")) {
                 return false;
             }
+            content_scanner_timeout = content_scanner_timeout_sec * 1000;
 
             if (findoptionS("scancleancache") == "off") {
                 scan_clean_cache = false;
@@ -532,12 +530,9 @@ bool OptionContainer::read(const char *filename, int type)
             return false;
         } // etc
         log_file_format = findoptionI("logfileformat");
-        if (!realitycheck(log_file_format, 1, 7, "logfileformat")) {
+        if (!realitycheck(log_file_format, 1, 6, "logfileformat")) {
             return false;
         } // etc
-
-	log_header_value = findoptionS("logheadervalue");
-
         if (findoptionS("anonymizelogs") == "on") {
             anonymise_logs = true;
         } else {
@@ -583,7 +578,6 @@ bool OptionContainer::read(const char *filename, int type)
             return false;
         }
         languagepath = findoptionS("languagedir") + "/" + findoptionS("language") + "/";
-        html_template_location = languagepath + "template.html";
 
         if (findoptionS("forwardedfor") == "on") {
             forwarded_for = true;
@@ -594,10 +588,10 @@ bool OptionContainer::read(const char *filename, int type)
         if (!realitycheck(log_exception_hits, 0, 2, "logexceptionhits")) {
             return false;
         }
-        if (findoptionS("createlistcachefiles") == "on") {
-            createlistcachefiles = true;
-        } else {
+        if (findoptionS("createlistcachefiles") == "off") {
             createlistcachefiles = false;
+        } else {
+            createlistcachefiles = true;
         }
         if (findoptionS("logconnectionhandlingerrors") == "on") {
             logconerror = true;
@@ -631,7 +625,14 @@ bool OptionContainer::read(const char *filename, int type)
         } else {
             log_client_hostnames = false;
         }
-
+	
+	if (log_client_hostnames == true && reverse_client_ip_lookups == false){
+            if (!is_daemonised) {
+                std::cerr << "logclienthostnames enabled but reverseclientiplookups disabled" << std::endl;
+            }
+            syslog(LOG_ERR, "logclienthostnames enabled but reverseclientiplookups disabled");
+            return false;
+	}
         if (findoptionS("recheckreplacedurls") == "on") {
             recheck_replaced_urls = true;
         } else {
@@ -649,7 +650,7 @@ bool OptionContainer::read(const char *filename, int type)
         filter_groups = findoptionI("filtergroups");
 
         if (((per_room_directory_location = findoptionS("perroomdirectory")) != "") || ((per_room_directory_location = findoptionS("perroomblockingdirectory")) != "")) {
-            loadRooms(true);
+            //loadRooms(true);
         }
 
         if (!realitycheck(filter_groups, 1, 0, "filtergroups")) {
@@ -751,8 +752,9 @@ bool OptionContainer::read(const char *filename, int type)
             filter_groups = 1;
 
         filter_groups_list_location = findoptionS("filtergroupslist");
-        std::string banned_ip_list_location(findoptionS("bannediplist"));
-        std::string exception_ip_list_location(findoptionS("exceptioniplist"));
+        banned_ip_list_location = findoptionS("bannediplist");
+        exception_ip_list_location = findoptionS("exceptioniplist");
+        group_names_list_location = findoptionS("groupnamesfile");
         std::string language_list_location(languagepath + "messages");
 
         if (filter_groups_list_location.length() == 0) {
@@ -760,10 +762,19 @@ bool OptionContainer::read(const char *filename, int type)
 #ifdef DGDEBUG
             std::cout << "Not using filtergroupslist" << std::endl;
 #endif
-        } else if (!doReadItemList(filter_groups_list_location.c_str(), &filter_groups_list, "filtergroupslist", true)) {
-            return false;
         } else {
             use_filter_groups_list = true;
+            if ((findoptionS("authrequiresuserandgroup") == "on") && (authplugins.size() > 1))
+                auth_requires_user_and_group = true;
+        }
+
+        if (group_names_list_location.length() == 0) {
+            use_group_names_list = false;
+#ifdef DGDEBUG
+            std::cout << "Not using groupnameslist" << std::endl;
+#endif
+        } else {
+            use_group_names_list = true;
         }
 
         if (findoptionS("prefercachedlists") == "on")
@@ -771,34 +782,22 @@ bool OptionContainer::read(const char *filename, int type)
         else
             prefer_cached_lists = false;
 
-        if (!exception_ip_list.readIPMelangeList(exception_ip_list_location.c_str())) {
-            std::cout << "Failed to read exceptioniplist" << std::endl;
-            return false;
-        }
-        if (!banned_ip_list.readIPMelangeList(banned_ip_list_location.c_str())) {
-            std::cout << "Failed to read bannediplist" << std::endl;
-            return false;
-        }
+        //if (!exception_ip_list.readIPMelangeList(exception_ip_list_location.c_str())) {
+        //    std::cout << "Failed to read exceptioniplist" << std::endl;
+        //    return false;
+        //}
+        //if (!banned_ip_list.readIPMelangeList(banned_ip_list_location.c_str())) {
+        //    std::cout << "Failed to read bannediplist" << std::endl;
+        //    return false;
+        //}
 
         if (!language_list.readLanguageList(language_list_location.c_str())) {
             return false;
         } // messages language file
 
-        if (reporting_level == 3) { // only if reporting set to HTML templ
-            if (!html_template.readTemplateFile(html_template_location.c_str())) {
-                if (!is_daemonised) {
-                    std::cerr << "Error reading HTML Template file: " << html_template_location << std::endl;
-                }
-                syslog(LOG_ERR, "Error reading HTML Template file: %s", html_template_location.c_str());
-                return false;
-                // HTML template file
-            }
-        }
 
-        if (!readFilterGroupConf()) {
-            if (!is_daemonised) {
+        if(!createLists(0))  {
                 std::cerr << "Error reading filter group conf file(s)." << std::endl;
-            }
             syslog(LOG_ERR, "%s", "Error reading filter group conf file(s).");
             return false;
         }
@@ -806,16 +805,16 @@ bool OptionContainer::read(const char *filename, int type)
 //post read filtergroup config checks - only for SLLMITM for now
 
 #ifdef _SSLMITM
-        bool ssl_mitm = false;
-        bool mitm_check_cert = false;
-        for (i = 0; i < numfg; i++) {
-            if (fg[i].ssl_mitm)
-                ssl_mitm = true;
-            if (fg[i].mitm_check_cert)
-                mitm_check_cert = true;
-        }
+        //bool ssl_mitm = false;
+        //bool mitm_check_cert = false;
+        //for (i = 0; i < numfg; i++) {
+            //if (fg[i].ssl_mitm)
+                //ssl_mitm = true;
+            //if (fg[i].mitm_check_cert)
+                //mitm_check_cert = true;
+        //}
 
-        if (ssl_mitm) {
+        if (enable_ssl) {
             if (ca_certificate_path != "") {
                 ca = new CertificateAuthority(ca_certificate_path.c_str(),
                     ca_private_key_path.c_str(),
@@ -879,7 +878,7 @@ bool OptionContainer::readinStdin()
 
 char *OptionContainer::inSiteList(String &url, ListContainer *lc, bool ip, bool ssl)
 {
-
+    String lastcategory;
     url.removeWhiteSpace(); // just in case of weird browser crap
     url.toLower();
     url.removePTP(); // chop off the ht(f)tp(s)://
@@ -889,7 +888,7 @@ char *OptionContainer::inSiteList(String &url, ListContainer *lc, bool ip, bool
     char *i;
     //bool isipurl = isIPHostname(url);
     while (url.contains(".")) {
-        i = lc->findInList(url.toCharArray());
+        i = lc->findInList(url.toCharArray(), lastcategory);
         if (i != NULL) {
             return i; // exact match
         }
@@ -897,7 +896,7 @@ char *OptionContainer::inSiteList(String &url, ListContainer *lc, bool ip, bool
     }
     if (url.length() > 1) { // allows matching of .tld
         url = "." + url;
-        i = lc->findInList(url.toCharArray());
+        i = lc->findInList(url.toCharArray(), lastcategory);
         if (i != NULL) {
             return i; // exact match
         }
@@ -910,6 +909,7 @@ char *OptionContainer::inURLList(String &url, ListContainer *lc, bool ip, bool s
 {
     unsigned int fl;
     char *i;
+    String lastcategory;
     String foundurl;
 #ifdef DGDEBUG
     std::cout << "inURLList: " << url << std::endl;
@@ -934,7 +934,7 @@ char *OptionContainer::inURLList(String &url, ListContainer *lc, bool ip, bool s
 #endif
     //  syslog(LOG_ERR, "inURLList (processed) url %s", url.c_str());
     while (url.before("/").contains(".")) {
-        i = lc->findStartsWith(url.toCharArray());
+        i = lc->findStartsWith(url.toCharArray(), lastcategory);
         if (i != NULL) {
             foundurl = i;
             fl = foundurl.length();
@@ -986,208 +986,6 @@ bool OptionContainer::doReadItemList(const char *filename, ListContainer *lc, co
     return true;
 }
 
-bool OptionContainer::inExceptionIPList(const std::string *ip, std::string *&host)
-{
-    return exception_ip_list.inList(*ip, host);
-}
-
-bool OptionContainer::inBannedIPList(const std::string *ip, std::string *&host)
-{
-    return banned_ip_list.inList(*ip, host);
-}
-
-// TODO: Filter rules should migrate to FOptionContainer.cpp ?  -- No, these are not filtergroup rules but nmaybe to their own cpp??
-
-bool OptionContainer::inRoom(const std::string &ip, std::string &room, std::string *&host, bool *block, bool *part_block, bool *isexception, String url)
-{
-    String temp;
-    char *ret;
-    for (std::list<struct room_item>::const_iterator i = rooms.begin(); i != rooms.end(); ++i) {
-        if (i->iplist->inList(ip, host)) {
-#ifdef DGDEBUG
-            std::cerr << " IP is in room: " << i->name << std::endl;
-#endif
-            temp = url;
-            ListContainer *lc;
-            if (i->sitelist) {
-                lc = i->sitelist;
-                if (inSiteList(temp, lc, false, false)) {
-#ifdef DGDEBUG
-                    std::cerr << " room site exception found: " << std::endl;
-#endif
-                    *isexception = true;
-                    room = i->name;
-                    return true;
-                }
-            }
-            temp = url;
-            if (i->urllist && inURLList(temp, i->urllist, false, false)) {
-#ifdef DGDEBUG
-                std::cerr << " room url exception found: " << std::endl;
-#endif
-                *isexception = true;
-                room = i->name;
-                return true;
-            }
-            if (i->block) {
-                *block = true;
-                *part_block = i->part_block;
-                room = i->name;
-#ifdef DGDEBUG
-                std::cerr << " room blanket block active: " << std::endl;
-#endif
-                return true;
-            } else {
-#ifdef DGDEBUG
-                std::cerr << " room - no url/site exception or block found: " << std::endl;
-#endif
-                return false;
-            }
-        }
-    }
-    return false;
-}
-
-// TODO: Filter rules should migrate to FOptionContainer.cpp ?
-
-void OptionContainer::loadRooms(bool throw_error)
-{
-    if (!throw_error && (per_room_directory_location == ""))
-        return;
-    DIR *d = opendir(per_room_directory_location.c_str());
-    if (d == NULL) {
-        if (throw_error) {
-            syslog(LOG_ERR, "Could not open room definitions directory: %s", strerror(errno));
-            std::cerr << "Could not open room definitions directory" << std::endl;
-            exit(1);
-        } else {
-            return;
-        }
-    }
-
-    struct dirent *f;
-    while ((f = readdir(d))) {
-        if (f->d_name[0] == '.')
-            continue;
-        std::string filename(per_room_directory_location);
-        filename.append(f->d_name);
-#ifdef DGDEBUG
-        std::cerr << " Room file found : " << filename.c_str() << std::endl;
-#endif
-        std::ifstream infile(filename.c_str(), std::ios::in);
-        if (!infile.good()) {
-            syslog(LOG_ERR, " Could not open file room definitions ");
-            std::cerr << " Could not open file room definitions: " << filename.c_str() << std::endl;
-            exit(1);
-        }
-#ifdef DGDEBUG
-        std::cerr << " Opened room file : " << filename.c_str() << std::endl;
-#endif
-
-        std::string roomname;
-#ifdef DGDEBUG
-        std::cerr << " Reading room file : " << filename.c_str() << std::endl;
-#endif
-        getline(infile, roomname);
-        if (infile.eof()) {
-            syslog(LOG_ERR, " Unexpected EOF ");
-            std::cerr << " Unexpected EOF: " << filename.c_str() << std::endl;
-            exit(1);
-        }
-        if (infile.fail()) {
-            syslog(LOG_ERR, " Unexpected failure on read");
-            std::cerr << " Unexpected failure on read: " << filename.c_str() << std::endl;
-            exit(1);
-        }
-        if (infile.bad()) {
-            syslog(LOG_ERR, " Unexpected badbit failure on read");
-            std::cerr << " Unexpected badbit failure on read: " << filename.c_str() << std::endl;
-            exit(1);
-        }
-        if (!infile.good()) {
-            syslog(LOG_ERR, " Could not open file room definitions ");
-            std::cerr << " Could not open file room definitions: " << filename.c_str() << std::endl;
-            exit(1);
-        }
-#ifdef DGDEBUG
-        std::cerr << " Room name is: " << roomname.c_str() << std::endl;
-#endif
-        roomname = roomname.substr(1);
-        room_item this_room;
-        this_room.name = roomname;
-        this_room.block = false;
-        this_room.part_block = false;
-        this_room.sitelist = NULL;
-        this_room.urllist = NULL;
-
-        IPList *contents = new IPList();
-        contents->ifsreadIPMelangeList(&infile, true, "#ENDLIST");
-        this_room.iplist = contents;
-        if (infile.eof()) { // is old style room block
-            this_room.block = true;
-            this_room.sitelist = NULL;
-            this_room.urllist = NULL;
-        } else {
-            std::string linestr;
-            String temp;
-            while (infile.good()) {
-                std::getline(infile, linestr);
-                if (infile.eof())
-                    break;
-                temp = linestr;
-                if (temp.startsWith("#SITELIST")) {
-                    ListContainer *sitelist = new ListContainer();
-                    if (sitelist->ifsReadSortItemList(&infile, true, "#ENDLIST", false, false, 0, filename.c_str())) {
-                        this_room.sitelist = sitelist;
-                    } else {
-                        delete sitelist;
-                    }
-                } else if (temp.startsWith("#URLLIST")) {
-                    ListContainer *urllist = new ListContainer();
-                    if (urllist->ifsReadSortItemList(&infile, true, "#ENDLIST", false, true, 0, filename.c_str())) {
-                        this_room.urllist = urllist;
-                    } else {
-                        delete urllist;
-                    }
-                } else if (temp.startsWith("#BLOCK")) {
-                    this_room.block = true;
-                }
-            }
-        }
-        if (this_room.block && (this_room.sitelist || this_room.urllist))
-            this_room.part_block = true;
-        rooms.push_back(this_room);
-        infile.close();
-        if (roomname.size() <= 2) {
-            if (!is_daemonised) {
-                std::cerr << "Could not read room from definitions file \"" << filename << '"' << std::endl;
-            }
-            syslog(LOG_ERR, "Could not read room from definitions file \"%s\"",
-                filename.c_str());
-            exit(1);
-        }
-        roomname = roomname.substr(1); // remove leading '#'
-    }
-
-    if (closedir(d) != 0) {
-        if (errno != EINTR) {
-            syslog(LOG_ERR, "Could not close room definitions directory: %s", strerror(errno));
-            exit(1);
-        }
-    }
-}
-
-void OptionContainer::deleteRooms()
-{
-    for (std::list<room_item>::iterator i = rooms.begin(); i != rooms.end(); ++i) {
-        delete i->iplist;
-        if (i->sitelist != NULL)
-            delete i->sitelist;
-        if (i->urllist != NULL)
-            delete i->urllist;
-    }
-    rooms.clear();
-}
 
 long int OptionContainer::findoptionI(const char *option)
 {
@@ -1276,112 +1074,14 @@ bool OptionContainer::realitycheck(long int l, long int minl, long int maxl, con
             // the console so we can write back an
             // error
 
-            std::cerr << "Config problem: " << emessage << " set to " << l << "; Value must be greater than " << minl << " and less than " << maxl << "." << std::endl;
+            std::cerr << "Config problem; check allowed values for " << emessage << std::endl;
         }
-        syslog(LOG_ERR, "Config problem %s set to %lu; Value must be greater than %lu and less than %lu.", emessage, l, minl, maxl);
+        syslog(LOG_ERR, "Config problem; check allowed values for %s", emessage);
         return false;
     }
     return true;
 }
 
-bool OptionContainer::readFilterGroupConf()
-{
-    String prefix(conffilename);
-    prefix = prefix.before(".conf");
-    prefix += "f";
-    String file;
-    String groupname;
-    bool need_html = false;
-    for (int i = 1; i <= filter_groups; i++) {
-        file = prefix + String(i);
-        file += ".conf";
-        if (!readAnotherFilterGroupConf(file.toCharArray(), groupname.toCharArray(), need_html)) {
-            if (!is_daemonised) {
-                std::cerr << "Error opening filter group config: " << file << std::endl;
-            }
-            syslog(LOG_ERR, "Error opening filter group config: %s", file.toCharArray());
-            return false;
-        }
-    }
-    if (!need_html && (reporting_level != 3)) {
-#ifdef DGDEBUG
-        std::cout << "Global reporting level not 3 & no filter groups using the template; so resetting it." << std::endl;
-#endif
-        html_template.reset();
-    }
-    return true;
-}
-
-bool OptionContainer::readAnotherFilterGroupConf(const char *filename, const char *groupname, bool &need_html)
-{
-#ifdef DGDEBUG
-    std::cout << "adding filter group: " << numfg << " " << filename << std::endl;
-#endif
-
-    // array of pointers to FOptionContainer
-    typedef FOptionContainer *PFOptionContainer;
-    FOptionContainer **temp = new PFOptionContainer[numfg + 1];
-    for (int i = 0; i < numfg; i++) {
-        temp[i] = fg[i];
-    }
-    if (numfg > 0) {
-        delete[] fg;
-    }
-    fg = temp;
-    fg[numfg] = new FOptionContainer;
-
-#ifdef DGDEBUG
-    std::cout << "added filter group: " << numfg << " " << filename << std::endl;
-#endif
-
-    // pass all the vars from OptionContainer needed
-    (*fg[numfg]).weighted_phrase_mode = weighted_phrase_mode;
-    (*fg[numfg]).force_quick_search = force_quick_search;
-    (*fg[numfg]).createlistcachefiles = createlistcachefiles;
-    (*fg[numfg]).reverse_lookups = reverse_lookups;
-
-    // pass in the group name
-    (*fg[numfg]).name = groupname;
-
-    // pass in the reporting level - can be overridden
-    (*fg[numfg]).reporting_level = reporting_level;
-
-#ifdef DGDEBUG
-    std::cout << "passed variables to filter group: " << numfg << " " << filename << std::endl;
-#endif
-
-    bool rc = (*fg[numfg]).read(filename);
-#ifdef DGDEBUG
-    std::cout << "read filter group: " << numfg << " " << filename << std::endl;
-#endif
-
-    numfg++;
-
-    if (!rc) {
-        return false;
-    }
-//<TODO> ifdef for ssl mitm
-#ifdef __SSLMITM
-    if (((fg[numfg - 1]->reporting_level == 3) || fg[numfg - 1]->ssl_mitm) && (html_template.html.size() == 0)) {
-#else
-    if ((fg[numfg - 1]->reporting_level == 3) && (html_template.html.size() == 0)) {
-#endif
-#ifdef DGDEBUG
-        std::cout << "One of the groups has overridden the reporting level! Loading the HTML template." << std::endl;
-#endif
-        need_html = true;
-        if (!html_template.readTemplateFile(html_template_location.c_str())) {
-            if (!is_daemonised) {
-                std::cerr << "Error reading HTML Template file: " << html_template_location << std::endl;
-            }
-            syslog(LOG_ERR, "Error reading HTML Template file: %s", html_template_location.c_str());
-            return false;
-            // HTML template file
-        }
-    }
-
-    return true;
-}
 
 bool OptionContainer::loadDMPlugins()
 {
@@ -1531,3 +1231,18 @@ bool OptionContainer::loadAuthPlugins()
     authplugins_end = authplugins.end();
     return true;
 }
+
+
+bool OptionContainer::createLists(int load_id)  {
+    std::shared_ptr<LOptionContainer> temp (new LOptionContainer(load_id));
+    if (temp->loaded_ok) {
+        current_LOC = temp;
+        return true;
+    }
+    return false;
+}
+
+
+std::shared_ptr<LOptionContainer> OptionContainer::currentLists()  {
+    return current_LOC;
+}
