@@ -19,39 +19,38 @@
 #include "RegExp.hpp"
 #include "Auth.hpp"
 #include "IPList.hpp"
+#include "Queue.hpp"
+#include "LOptionContainer.hpp"
 
 #include <deque>
+#include <atomic>
 
 #ifdef __SSLMITM
 #include "CertificateAuthority.hpp"
 #endif
 
+
 // DECLARATIONS
-struct room_item {
-    std::string name;
-    IPList *iplist;
-    ListContainer *sitelist;
-    ListContainer *urllist;
-    bool block;
-    bool part_block;
-};
 
 class OptionContainer
 {
     public:
+    Queue<std::string>* log_Q;
+    Queue<Socket*>* http_worker_Q;
+
     // all our many, many options
     int filter_groups;
     int log_exception_hits;
     bool non_standard_delimiter;
     int log_file_format;
+    std::string log_header_value;
+    std::string ident_header_value;
     int weighted_phrase_mode; // PIP added in - not sure if still required
     bool show_weighted_found;
     bool forwarded_for;
     bool createlistcachefiles;
     bool use_custom_banned_image;
     std::string custom_banned_image_file;
-    std::string log_header_value;
-    std::string ident_header_value;
     bool use_custom_banned_flash;
     std::string custom_banned_flash_file;
     bool reverse_lookups;
@@ -66,6 +65,7 @@ class OptionContainer
     int url_cache_age;
     int phrase_filter_mode;
     int preserve_case;
+    int max_header_lines;
     bool hex_decode_content;
     bool force_quick_search;
     bool map_auth_to_ports;
@@ -80,17 +80,14 @@ class OptionContainer
     bool get_orig_ip;
 #endif
     int ll;
-    int max_children;
     int proxy_timeout;
+    int proxy_timeout_sec;
     int proxy_failure_log_interval;
     int exchange_timeout;
+    int exchange_timeout_sec;
     int pcon_timeout;
-    int min_children;
-    int maxspare_children;
-    int prefork_children;
-    int minspare_children;
-    int maxage_children;
-    int gentle_chunk;
+    int pcon_timeout_sec;
+    int http_workers;
     std::string daemon_user_name;
     std::string daemon_group_name;
     int proxy_user;
@@ -99,13 +96,18 @@ class OptionContainer
     int max_ips;
     bool recheck_replaced_urls;
     bool use_filter_groups_list;
+    bool use_group_names_list;
     bool auth_needs_proxy_query;
+    bool auth_requires_user_and_group;
     bool total_block_site_flag;
     bool total_block_url_flag;
+    bool enable_ssl;
 
     bool prefer_cached_lists;
     std::string languagepath;
     std::string filter_groups_list_location;
+    std::string banned_ip_list_location;
+    std::string exception_ip_list_location;
     std::string log_location;
     std::string stat_location;
     std::string ipc_filename;
@@ -117,20 +119,17 @@ class OptionContainer
     bool monitor_helper_flag;
     std::string monitor_flag_prefix;
     bool monitor_flag_flag;
-    int monitor_start; // call monitorhelper start and/or set monitor_flag file when number of freechildren is more than this figure
     std::string dstat_location;
     bool dstat_log_flag;
+    bool stats_human_readable;
     int dstat_interval;
+    bool dns_user_logging;
+    std::string dns_user_logging_domain;
 
     // Hardware/organisation/etc. IDs
     std::string logid_1;
     std::string logid_2;
 
-#ifdef SG_LOGFORMAT
-    // Product ID
-    std::string prod_id;
-#endif
-
     bool no_daemon;
     bool no_logger;
     bool log_syslog;
@@ -173,11 +172,12 @@ class OptionContainer
     int initial_trickle_delay;
     int trickle_delay;
     int content_scanner_timeout;
+    int content_scanner_timeout_sec;
 
     HTMLTemplate html_template;
     ListContainer filter_groups_list;
-    IPList exception_ip_list;
-    IPList banned_ip_list;
+    //IPList exception_ip_list;
+    //IPList banned_ip_list;
     LanguageContainer language_list;
     ImageContainer banned_image;
     ImageContainer banned_flash;
@@ -193,7 +193,7 @@ class OptionContainer
     std::deque<Plugin *>::iterator authplugins_end;
 
     ListManager lm;
-    FOptionContainer **fg;
+   // FOptionContainer **fg;
     int numfg;
 
     // access denied domain (when using the CGI)
@@ -203,26 +203,21 @@ class OptionContainer
     bool loadCSPlugins();
     bool loadAuthPlugins();
     void deletePlugins(std::deque<Plugin *> &list);
-    void deleteFilterGroups();
-    void deleteFilterGroupsJustListData();
+ //   void deleteFilterGroups();
+  //  void deleteFilterGroupsJustListData();
 
     //...and the functions that read them
 
     OptionContainer();
     ~OptionContainer();
-    bool read(const char *filename, int type);
+    bool read(std::string& filename, int type);
     void reset();
-    bool inExceptionIPList(const std::string *ip, std::string *&host);
-    bool inBannedIPList(const std::string *ip, std::string *&host);
-    bool readFilterGroupConf();
+  //  bool inExceptionIPList(const std::string *ip, std::string *&host);
+  //  bool inBannedIPList(const std::string *ip, std::string *&host);
+    //bool readFilterGroupConf();
     // public so fc_controlit can reload filter group config files
     bool doReadItemList(const char *filename, ListContainer *lc, const char *fname, bool swsort);
 
-    // per-room blocking and URL whitelisting: see if given IP is in a room; if it is, return true and put the room name in "room"
-    bool inRoom(const std::string &ip, std::string &room, std::string *&host, bool *block, bool *part_block, bool *isexception, String url);
-    void loadRooms(bool throw_error);
-    void deleteRooms();
-
     char *inSiteList(String &url, ListContainer *lc, bool swsort, bool ip);
     char *inURLList(String &url, ListContainer *lc, bool swsort, bool ip);
 
@@ -230,16 +225,22 @@ class OptionContainer
     bool readinStdin();
     bool inTotalBlockList(String &url);
     bool use_total_block_list;
-
-    private:
     std::string per_room_directory_location;
-    std::deque<std::string> conffile;
-    String conffilename;
-    int reporting_level;
+    bool createLists(int load_id);
+    std::shared_ptr<LOptionContainer> currentLists();
+    std::atomic<int> LC_cnt;
 
+    //LOptionContainer* current_LOC;
+    std::shared_ptr<LOptionContainer> current_LOC;
+    std::string conffilename;
     std::string html_template_location;
     std::string group_names_list_location;
 
+    private:
+    std::deque<std::string> conffile;
+    int reporting_level;
+
+
     bool loadDMPlugins();
 
     bool precompileregexps();
@@ -249,8 +250,7 @@ class OptionContainer
     bool readAnotherFilterGroupConf(const char *filename, const char *groupname, bool &need_html);
     std::deque<String> findoptionM(const char *option);
 
-    bool inIPList(const std::string *ip, ListContainer &list, std::string *&host);
-    std::list<room_item> rooms;
+//    bool inIPList(const std::string *ip, ListContainer &list, std::string *&host);
 };
 
 #endif
