@@ -34,7 +34,8 @@
 extern bool reloadconfig;
 
 #ifndef X509_V_FLAG_TRUSTED_FIRST
-#warning "X509_V_FLAG_TRUSTED_FIRST not available, chain creation will be unreliable"
+#warning "openssl X509_V_FLAG_TRUSTED_FIRST not available, certificate chain creation will be unreliable and will fail on some sites"
+#warning "To fix install a later version of openssl"
 #define X509_V_FLAG_TRUSTED_FIRST 0
 #endif
 #endif
@@ -45,27 +46,30 @@ extern bool reloadconfig;
 Socket::Socket()
 {
     sck = socket(AF_INET, SOCK_STREAM, 0);
+    if (sck < 0) {
+        s_errno = errno;
+    } else {
+        memset(&my_adr, 0, sizeof my_adr);
+        memset(&peer_adr, 0, sizeof peer_adr);
+        my_adr.sin_family = AF_INET;
+        peer_adr.sin_family = AF_INET;
+        peer_adr_length = sizeof(struct sockaddr_in);
+        int f = 1;
 
-    memset(&my_adr, 0, sizeof my_adr);
-    memset(&peer_adr, 0, sizeof peer_adr);
-    my_adr.sin_family = AF_INET;
-    peer_adr.sin_family = AF_INET;
-    peer_adr_length = sizeof(struct sockaddr_in);
-    int f = 1;
+        if (sck > 0)
+            int res = setsockopt(sck, IPPROTO_TCP, TCP_NODELAY, &f, sizeof(int));
 
-    if (sck > 0)
-        int res = setsockopt(sck, IPPROTO_TCP, TCP_NODELAY, &f, sizeof(int));
-
-    my_port = 0;
+        my_port = 0;
 
 #ifdef __SSLMITM
-    ssl = NULL;
-    ctx = NULL;
-    isssl = false;
-    issslserver = false;
+        ssl = NULL;
+        ctx = NULL;
+        isssl = false;
+        issslserver = false;
 #else
-    isssl = false;
+        isssl = false;
 #endif
+    }
 }
 
 // create socket from pre-existing FD (address structs will be invalid!)
@@ -80,7 +84,7 @@ Socket::Socket(int fd)
     int f = 1;
 
     int res = setsockopt(sck, IPPROTO_TCP, TCP_NODELAY, &f, sizeof(int));
-
+    if (res < 0) s_errno = errno;
     my_port = 0;
 
 #ifdef __SSLMITM
@@ -107,7 +111,7 @@ Socket::Socket(int newfd, struct sockaddr_in myip, struct sockaddr_in peerip)
     int f = 1;
 
     int res = setsockopt(sck, IPPROTO_TCP, TCP_NODELAY, &f, sizeof(int));
-
+    if (res < 0) s_errno = errno;
     my_port = 0;
 
 #ifdef __SSLMITM
@@ -158,12 +162,18 @@ void Socket::reset()
     this->baseReset();
 
     sck = socket(AF_INET, SOCK_STREAM, 0);
+    if (sck < 0) {
+        s_errno = errno;
+        return;
+    }
 
     memset(&my_adr, 0, sizeof my_adr);
     memset(&peer_adr, 0, sizeof peer_adr);
     my_adr.sin_family = AF_INET;
     peer_adr.sin_family = AF_INET;
     peer_adr_length = sizeof(struct sockaddr_in);
+    infds[0].fd = sck;
+    outfds[0].fd = sck;
 
 #ifdef __SSLMITM
     if (isssl) {
@@ -175,12 +185,50 @@ void Socket::reset()
 // connect to given IP & port (following default constructor)
 int Socket::connect(const std::string &ip, int port)
 {
+    //if (sck > -1)  reset();   // just in case this is called with socket still open
+    reset();   // do it anyway as we need sck to be allocated
+
+    if (sck < 0) // socket creation error
+    { return -1;
+    }
+
     int len = sizeof my_adr;
     peer_adr.sin_port = htons(port);
     inet_aton(ip.c_str(), &peer_adr.sin_addr);
     my_port = port;
-
-    return ::connect(sck, (struct sockaddr *)&peer_adr, len);
+    // make non-blocking for connect only so that we can timeout connect
+    fcntl(sck, F_SETFL, O_NONBLOCK);
+    s_errno = 0;
+    errno = 0;
+    int ret = ::connect(sck, (struct sockaddr *)&peer_adr, len);
+    if (ret < 0 && errno == EINPROGRESS) ret = 0;
+    else s_errno = errno;
+    if (ret == 0){
+        int rc = poll(outfds,1,timeout);
+        if (rc == 0) {
+            timedout = true;
+            ret = -1;
+        }
+        else if (rc < 0) {
+            s_errno = errno;
+            ret = -1;
+        }
+        else {  // ret == 1
+            int so_error;
+            socklen_t len = sizeof so_error;
+            getsockopt(sck, SOL_SOCKET, SO_ERROR, &so_error, &len);
+            if (so_error != 0) {
+                sockerr = true;
+                s_errno = so_error;
+                ret = -1;
+            } else {
+                ret = 0;
+            }
+        }
+    }
+    fcntl(sck, F_SETFL, 0);  // make blocking again
+    if (ret < 0) close();
+    return ret;
 }
 // bind socket to given port
 int Socket::bind(int port)
@@ -215,14 +263,19 @@ int Socket::bind(const std::string &ip, int port)
 Socket *Socket::accept()
 {
     peer_adr_length = sizeof(struct sockaddr_in);
-    int newfd = this->baseAccept((struct sockaddr *)&peer_adr, &peer_adr_length);
+    s_errno = 0;
+    errno = 0;
+//    int newfd = this->baseAccept((struct sockaddr *)&peer_adr, &peer_adr_length);
+    int newfd = ::accept(sck,( struct sockaddr *)&peer_adr, &peer_adr_length);
 
     if (newfd > 0) {
         Socket *s = new Socket(newfd, my_adr, peer_adr);
         s->setPort(my_port);
         return s;
-    } else
+    } else {
+        s_errno = errno;
         return NULL;
+    }
 }
 
 #ifdef __SSLMITM
@@ -249,6 +302,8 @@ int Socket::startSslClient(const std::string &certificate_path, String hostname)
 
     //set the timeout for the ssl session
     if (SSL_CTX_set_timeout(ctx, 130l) < 1) {
+            SSL_CTX_free(ctx);
+            ctx = NULL;
         return -1;
     }
 
@@ -268,7 +323,7 @@ int Socket::startSslClient(const std::string &certificate_path, String hostname)
     } else if (!SSL_CTX_set_default_verify_paths(ctx)) //use default if no certPpath given
     {
 #ifdef DGDEBUG
-        std::cout << "couldnt load default certificates" << std::endl;
+        std::cout << "couldnt load certificates" << std::endl;
 #endif
             log_ssl_errors("couldnt load default certificates for %s", hostname.c_str());
         //tidy up
@@ -282,7 +337,9 @@ int Socket::startSslClient(const std::string &certificate_path, String hostname)
     X509_VERIFY_PARAM *x509_param = X509_VERIFY_PARAM_new();
     if (!x509_param) {
         log_ssl_errors("couldnt add validation params for %s", hostname.c_str());
-        X509_VERIFY_PARAM_free(x509_param);
+        //X509_VERIFY_PARAM_free(x509_param);
+            SSL_CTX_free(ctx);
+            ctx = NULL;
         return -2;
     }
 
@@ -290,6 +347,8 @@ int Socket::startSslClient(const std::string &certificate_path, String hostname)
     if (!X509_VERIFY_PARAM_set_flags(x509_param, X509_V_FLAG_TRUSTED_FIRST)) {
         log_ssl_errors("couldnt add validation params for %s", hostname.c_str());
         X509_VERIFY_PARAM_free(x509_param);
+            SSL_CTX_free(ctx);
+            ctx = NULL;
         return -2;
     }
 
@@ -297,10 +356,12 @@ int Socket::startSslClient(const std::string &certificate_path, String hostname)
     if (!SSL_CTX_set1_param(ctx, x509_param)) {
         log_ssl_errors("couldnt add validation params for %s", hostname.c_str());
         X509_VERIFY_PARAM_free(x509_param);
+            SSL_CTX_free(ctx);
+            ctx = NULL;
         return -2;
     }
 
-    X509_VERIFY_PARAM_free(x509_param);
+    X509_VERIFY_PARAM_free(x509_param);     // try not freeing this as SSL_CTX_free seems to be ring to free it
 
     //hand socket over to ssl lib
     ERR_clear_error();
@@ -321,7 +382,6 @@ int Socket::startSslClient(const std::string &certificate_path, String hostname)
     if (rc < 0) {
         //ERR_print_errors_fp(stderr);
         log_ssl_errors("ssl_connect failed to %s", hostname.c_str());
-//printerr(strerror(errno));
 #ifdef DGDEBUG
         std::cout << "ssl_connect failed with error " << SSL_get_error(ssl, rc) << std::endl;
 #endif
@@ -355,6 +415,7 @@ void Socket::stopSsl()
 #ifdef DGDEBUG
     std::cout << "ssl stopping" << std::endl;
 #endif
+    if(!isssl) return;
 
     isssl = false;
 
@@ -688,6 +749,9 @@ bool Socket::checkForInput()
         return true;
     }
 
+    if (!BaseSocket::checkForInput())
+        return false;
+
     //see if we can do an ssl read of 1 byte
 //    char buf[1];
 
@@ -708,54 +772,52 @@ bool Socket::checkForInput()
     return true;
 }
 
-// blocking check for waiting data - blocks for up to given timeout, can be told to break on signal-triggered config reloads
-void Socket::checkForInput(int timeout, bool honour_reloadconfig) throw(std::exception)
+bool Socket::bcheckForInput(int timeout)
 {
     if (!isssl) {
-        BaseSocket::checkForInput(timeout, honour_reloadconfig);
-        return;
+        return BaseSocket::bcheckForInput(timeout);
     }
-    //cant do this on a blocking ssl socket as far as i can work out
-
-    return;
-}
-
-bool Socket::readyForOutput()
-{
-    if (!isssl) {
-        return BaseSocket::readyForOutput();
-    }
-
-    //cant do this on a blocking ssl socket as far as i can work out
-
     return true;
 }
 
-void Socket::readyForOutput(int timeout, bool honour_reloadconfig) throw(std::exception)
+
+bool Socket::readyForOutput()
 {
-    if (!isssl) {
-        BaseSocket::readyForOutput(timeout, honour_reloadconfig);
-        return;
-    }
+    //if (!isssl) {
+        return BaseSocket::readyForOutput();
+    //}
 
     //cant do this on a blocking ssl socket as far as i can work out
 
-    return;
+    //return true;
 }
 
+bool Socket::breadyForOutput(int timeout)
+{
+    //if (!isssl) {
+        return BaseSocket::breadyForOutput(timeout);
+    //}
+    //return true;
+}
+
+
 // read a line from the socket, can be told to break on config reloads
 int Socket::getLine(char *buff, int size, int timeout, bool honour_reloadconfig, bool *chopped, bool *truncated) throw(std::exception)
 {
     if (!isssl) {
         return BaseSocket::getLine(buff, size, timeout, honour_reloadconfig, chopped, truncated);
     }
+#ifdef DGDEBUG
+		std::cout << "in getline (SSL): timeout: " << timeout << " size: " << size << std::endl;
+
+#endif
 
     // first, return what's left from the previous buffer read, if anything
     int i = 0;
     if ((bufflen - buffstart) > 0) {
-        /*#ifdef DGDEBUG
-		std::cout << "data already in buffer; bufflen: " << bufflen << " buffstart: " << buffstart << std::endl;
-#endif*/
+#ifdef DGDEBUG
+		std::cout << "getline(ssl) data already in buffer; bufflen: " << bufflen << " buffstart: " << buffstart << std::endl;
+#endif
         int tocopy = size - 1;
         if ((bufflen - buffstart) < tocopy)
             tocopy = bufflen - buffstart;
@@ -775,28 +837,37 @@ int Socket::getLine(char *buff, int size, int timeout, bool honour_reloadconfig,
     while (i < (size - 1)) {
         buffstart = 0;
         bufflen = 0;
-        //try {
-            //checkForInput(timeout, honour_reloadconfig);
-        //} catch (std::exception &e) {
-            //throw std::runtime_error(std::string("Can't read from socket: ") + strerror(errno)); // on error
-        //}
-        bufflen = SSL_read(ssl, buffer, 1024);
+ //       try {
+ //            checkForInput(timeout, honour_reloadconfig);
+ //       } catch (std::exception &e) {
+//            throw std::runtime_error(std::string("Can't read from socket: ") + strerror(errno)); // on error
+ //       }
+ //       if( bcheckSForInput(timeout))
+            bufflen = SSL_read(ssl, buffer, 4096);
 #ifdef DGDEBUG
-//std::cout << "read into buffer; bufflen: " << bufflen <<std::endl;
+std::cout << "getline (SSL) read into buffer; bufflen: " << bufflen <<std::endl;
 #endif
         if (bufflen < 0) {
-            if (errno == EINTR && (honour_reloadconfig ? !reloadconfig : true)) {
-                continue;
-            }
-         //   std::cout << "SSL_read failed with error " << SSL_get_error(ssl, bufflen) << std::endl;
+  //          if (errno == EINTR ) {
+   //             continue;
+    //        }
+
+#ifdef DGDEBUG
+            std::cerr << "SSL_read failed with error " << SSL_get_error(ssl, bufflen) << std::endl;
+#endif
             log_ssl_errors("ssl_read failed %s", "");
-            throw std::runtime_error(std::string("Can't read from ssl socket")); //strerror(errno));  // on error
+            return -1;
+//            throw std::runtime_error(std::string("Can't read from ssl socket")); //strerror(errno));  // on error
         }
         //if socket closed...
         if (bufflen == 0) {
             buff[i] = '\0'; // ...terminate string & return what read
             if (truncated)
                 *truncated = true;
+#ifdef DGDEBUG
+            if (truncated)
+            	std::cout << "Getline(SSL) truncated buffer end reached before we found a newline: bufflen == 0" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;;
+#endif
             return i;
         }
         int tocopy = bufflen;
@@ -804,6 +875,9 @@ int Socket::getLine(char *buff, int size, int timeout, bool honour_reloadconfig,
             tocopy = (size - 1) - i;
         char *result = (char *)memccpy(buff + i, buffer, '\n', tocopy);
         if (result != NULL) {
+#ifdef DGDEBUG
+        std::cout << "getLine result (SSL): " << result << std::endl;
+#endif
             // indicate that a newline was chopped off, if desired
             if (chopped)
                 *chopped = true;
@@ -812,30 +886,31 @@ int Socket::getLine(char *buff, int size, int timeout, bool honour_reloadconfig,
             return i + (result - (buff + i));
         }
         i += tocopy;
+#ifdef DGDEBUG
+            std::cout << "getLine(SSL) buffer =: " << buff[i] << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+#endif
     }
     // oh dear - buffer end reached before we found a newline
     buff[i] = '\0';
     if (truncated)
         *truncated = true;
+#ifdef DGDEBUG
+    	if (truncated)
+            std::cout << "Getline(SSL) truncated buffer end reached before we found a newline: " << buff  << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;;
+#endif
     return i;
 }
 
 // write line to socket
-void Socket::writeString(const char *line) throw(std::exception)
+bool Socket::writeString(const char *line) //throw(std::exception)
 {
     int l = strlen(line);
-    if (!writeToSocket(line, l, 0, timeout)) {
-        throw std::runtime_error(std::string("Can't write to socket: ") + strerror(errno));
-    }
+    return writeToSocket(line, l, 0, timeout);
+//    if (!writeToSocket(line, l, 0, timeout)) {
+ //       throw std::runtime_error(std::string("Can't write to socket: ") + strerror(errno));
+    //}
 }
 
-// write data to socket - throws exception on failure, can be told to break on config reloads
-void Socket::writeToSockete(const char *buff, int len, unsigned int flags, int timeout, bool honour_reloadconfig) throw(std::exception)
-{
-    if (!writeToSocket(buff, len, flags, timeout, honour_reloadconfig)) {
-        throw std::runtime_error(std::string("Can't write to socket: ") + strerror(errno));
-    }
-}
 
 // write data to socket - can be told not to do an initial readyForOutput, and to break on config reloads
 bool Socket::writeToSocket(const char *buff, int len, unsigned int flags, int timeout, bool check_first, bool honour_reloadconfig)
@@ -847,23 +922,26 @@ bool Socket::writeToSocket(const char *buff, int len, unsigned int flags, int ti
     int actuallysent = 0;
     int sent;
     while (actuallysent < len) {
-        //if (check_first) {
-         //   try {
-               //readyForOutput(timeout, honour_reloadconfig); // throws exception on error or timeout
-            //} catch (std::exception &e) {
-                //return false;
-            //}
-        //}
+       if (check_first) {
+    //        try {
+                if(!breadyForOutput(timeout))
+                   return false;
+     //       } catch (std::exception &e) {
+      //          return false;
+       //     }
+        }
         ERR_clear_error();
         sent = SSL_write(ssl, buff + actuallysent, len - actuallysent);
         if (sent < 0) {
-            if (errno == EINTR && (honour_reloadconfig ? !reloadconfig : true)) {
-                continue; // was interupted by signal so restart
-            }
+    //        if (errno == EINTR ) {
+    //            continue; // was interupted by signal so restart
+    //        }
+            s_errno = errno;
             log_ssl_errors("ssl_write failed %s", "");
             return false;
         }
         if (sent == 0) {
+            ishup = true;
             return false; // other end is closed
         }
         actuallysent += sent;
@@ -874,6 +952,10 @@ bool Socket::writeToSocket(const char *buff, int len, unsigned int flags, int ti
 // read a specified expected amount and return what actually read
 int Socket::readFromSocketn(char *buff, int len, unsigned int flags, int timeout)
 {
+    return readFromSocket(buff, len, flags, timeout, true, false);
+
+
+#ifdef NODEF
     if (!isssl) {
         return BaseSocket::readFromSocketn(buff, len, flags, timeout);
     }
@@ -898,11 +980,11 @@ int Socket::readFromSocketn(char *buff, int len, unsigned int flags, int timeout
     }
 
     while (cnt > 0) {
-        try {
-            checkForInput(timeout); // throws exception on error or timeout
-        } catch (std::exception &e) {
-            return -1;
-        }
+    //    try {
+    //        bcheckSForInput(timeout);        //  this may be wrong - why is data not being read into socket buffer????
+    //    } catch (std::exception &e) {
+     //       return -1;
+     //   }
         ERR_clear_error();
         rc = SSL_read(ssl, buff, cnt);
 #ifdef DGDEBUG
@@ -910,19 +992,22 @@ int Socket::readFromSocketn(char *buff, int len, unsigned int flags, int timeout
 #endif
 
         if (rc < 0) {
-            if (errno == EINTR) {
-                continue;
-            }
+       //     if (errno == EINTR) {
+         //       continue;
+           // }
             log_ssl_errors("ssl_read failed %s", "");
+           s_errno = errno;
             return -1;
         }
         if (rc == 0) { // eof
+             ishup = true;
             return len - cnt;
         }
         buff += rc;
         cnt -= rc;
     }
     return len;
+#endif
 }
 
 // read what's available and return error status - can be told not to do an initial checkForInput, and to break on reloads
@@ -937,7 +1022,7 @@ int Socket::readFromSocket(char *buff, int len, unsigned int flags, int timeout,
     int tocopy = 0;
     if ((bufflen - buffstart) > 0) {
 #ifdef DGDEBUG
-        std::cout << "Socket::readFromSocket: data already in buffer; bufflen: " << bufflen << " buffstart: " << buffstart << std::endl;
+        std::cout << "Socket::readFromSocket(ssl): data already in buffer; bufflen: " << bufflen << " buffstart: " << buffstart << std::endl;
 #endif
         tocopy = len;
         if ((bufflen - buffstart) < len)
@@ -950,28 +1035,79 @@ int Socket::readFromSocket(char *buff, int len, unsigned int flags, int timeout,
             return len;
     }
 
+#ifdef DGDEBUG
+        std::cout << "Socket::readFromSocket(ssl): now entering loop to read: " << cnt << " bytes: " << std::endl;
+#endif
     int rc;
-    //if (check_first) {
-        //try {
-            //checkForInput(timeout, honour_reloadconfig);
-        //} catch (std::exception &e) {
-            //return -1;
-        //}
-    //}
-    while (true) {
+    while (cnt > 0) {
+ //   if (check_first) {
+//#ifdef DGDEBUG
+//       std::cout << "Socket::readFromSocket(ssl): before bcheckSForInput timeout is: " << timeout  << std::endl;
+//#endif
+//          if(!bcheckSForInput(timeout))
+  //          return -1;
+//#ifdef DGDEBUG
+//        std::cout << "Socket::readFromSocket(ssl): after bcheckSForInput OK return: " << std::endl;
+//#endif
+  // }
+//    while (true)
+        bool inbuffer;
+#ifdef DGDEBUG
+        std::cout << "Socket::readFromSocket(ssl): before SSL_read: "  << std::endl;
+#endif
         ERR_clear_error();
-        rc = SSL_read(ssl, buff, cnt);
+       if (true) {     // was if(cnt > 4095)
+            inbuffer = false;
+           rc = SSL_read(ssl, buff, cnt);        //  data larger than SSL buffer so ok to read directly into output buffer
+        } else {
+            inbuffer = true;
+           rc = SSL_read(ssl, buffer, 4096);   // read into socket buffer to flush SSL buffer
+        }
 
         if (rc < 0) {
-            if (errno == EINTR && (honour_reloadconfig ? !reloadconfig : true)) {
-                continue;
-            }
+            s_errno = errno;
             log_ssl_errors("ssl_read failed %s", "");
+#ifdef DGDEBUG
+        std::cout << "ssl_read failed" << s_errno << " failed to read " << cnt << " bytes" << std::endl;
+#endif
+            rc = 0;
         }
-        break;
+        if (rc == 0) { // eof
+             ishup = true;
+             return len - cnt;
+             }
+
+        if (inbuffer) {
+#ifdef DGDEBUG
+        std::cout << "Inbuffer SSL read to return " << cnt << " bytes" << std::endl;
+#endif
+
+           buffstart = 0;
+           bufflen = rc;
+           if ((bufflen - buffstart) > 0) {
+              tocopy = cnt;
+              if ((bufflen - buffstart) < cnt)
+              tocopy = bufflen - buffstart;
+              memcpy(buff, buffer + buffstart, tocopy);
+              cnt -= tocopy;
+              buffstart += tocopy;
+              buff += tocopy;
+#ifdef DGDEBUG
+        std::cout << "Inbuffer SSL read returned " << tocopy << " bytes" << std::endl;
+#endif
+           }
+         } else {
+        buff += rc;
+        cnt -= rc;
+         }
+ //       break;
+#ifdef DGDEBUG
+        std::cout << "Still to read (SSL) " << cnt << " bytes" << std::endl;
+#endif
     }
 
-    return rc + tocopy;
+//    return rc + tocopy;
+      return len;
 }
 
 #endif //__SSLMITM
