@@ -12,6 +12,7 @@
 #include "String.hpp"
 
 #include <deque>
+#include<thread>
 
 // DECLARATIONS
 
@@ -41,6 +42,9 @@ class SocketArray
     // shove all socket FDs into the given array (pass in unallocated)
     int *getFDAll();
 
+    // try connecting to all our sockets which are still open to allow tidy close
+    void self_connect();
+
     // array dereference operator
     Socket *operator[](int i)
     {
@@ -50,6 +54,7 @@ class SocketArray
     private:
     // our sock collection container
     Socket *drawer;
+    std::thread* threads;
     // how many sockets we have
     unsigned int socknum;
 };
