@@ -15,7 +15,7 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 // INCLUDES
 
@@ -75,8 +75,8 @@ class dnsauthinstance : public AuthPlugin
             is_connection_based = true;
     };
 
-    int identify(Socket &peercon, Socket &proxycon, HTTPHeader &h, std::string &string);
-    int determineGroup(std::string &user, int &fg);
+    int identify(Socket &peercon, Socket &proxycon, HTTPHeader &h, std::string &string, bool &is_real_user);
+    int determineGroup(std::string &user, int &fg, ListContainer &uglc);
 
     int init(void *args);
     int quit();
@@ -153,7 +153,7 @@ int dnsauthinstance::init(void *args)
 // DNS-AUTH filter group determination
 // never actually return NOUSER from this, because we don't actually look in the filtergroupslist.
 // NOUSER stops ConnectionHandler from querying subsequent plugins.
-int dnsauthinstance::identify(Socket &peercon, Socket &proxycon, HTTPHeader &h, /*int &fg,*/ std::string &string)
+int dnsauthinstance::identify(Socket &peercon, Socket &proxycon, HTTPHeader &h, /*int &fg,*/ std::string &string, bool &is_real_user)
 {
     String p1, p2, ippath;
 
@@ -182,6 +182,7 @@ int dnsauthinstance::identify(Socket &peercon, Socket &proxycon, HTTPHeader &h,
 #endif
     if (getdnstxt(ippath)) {
         string = userst.user;
+        is_real_user = true;
         return DGAUTH_OK;
     } else {
         // redirect code
@@ -204,7 +205,7 @@ int dnsauthinstance::identify(Socket &peercon, Socket &proxycon, HTTPHeader &h,
     }
 }
 
-int dnsauthinstance::determineGroup(std::string &user, int &fg)
+int dnsauthinstance::determineGroup(std::string &user, int &fg, ListContainer &uglc)
 {
     fg = userst.group;
 #ifdef DGDEBUG
@@ -260,7 +261,7 @@ bool dnsauthinstance::getdnstxt(String &ippath)
                 for (unsigned int j1 = 1; j1 < ns_rr_rdlen(rr); j1++) {
                     p[j++] = k[j1];
                 }
-                p[j] = (char)NULL;
+                p[j] = '\0';
 #ifdef DGDEBUG
                 std::cout << "ns_rr_data returned " << p << std::endl;
 #endif
@@ -301,17 +302,18 @@ bool dnsauthinstance::inAuthByPassLists(HTTPHeader &h)
 {
     String url = h.url();
     String urld = h.decode(url);
+    FOptionContainer* foc = o.currentLists()->fg[0];
     url.removePTP();
     if (url.contains("/")) {
         url = url.before("/");
     }
-    bool is_ip = (*o.fg[0]).isIPHostname(url);
+    bool is_ip = (*foc).isIPHostname(url);
     bool is_ssl = h.requestType() == "CONNECT";
 
-    if ((*o.fg[0]).inAuthExceptionSiteList(urld, true, is_ip, is_ssl)) {
+    if ((*foc).inAuthExceptionSiteList(urld, true, is_ip, is_ssl)) {
         //						exceptioncat = (*o.lm.l[(*o.fg[filtergroup]).exception_site_list]).lastcategory.toCharArray();
         return true;
-    } else if ((*o.fg[0]).inAuthExceptionURLList(urld, true, is_ip, is_ssl)) {
+    } else if ((*foc).inAuthExceptionURLList(urld, true, is_ip, is_ssl)) {
         //					exceptioncat = (*o.lm.l[(*o.fg[filtergroup]).exception_url_list]).lastcategory.toCharArray();
         return true;
     }
