@@ -78,8 +78,8 @@ class ipinstance : public AuthPlugin
             is_connection_based = true;
     };
 
-    int identify(Socket &peercon, Socket &proxycon, HTTPHeader &h, std::string &string);
-    int determineGroup(std::string &user, int &fg);
+    int identify(Socket &peercon, Socket &proxycon, HTTPHeader &h, std::string &string, bool &is_real_user);
+    int determineGroup(std::string &user, int &fg, ListContainer &uglc);
 
     int init(void *args);
     int quit();
@@ -130,7 +130,7 @@ int ipinstance::init(void *args)
         return readIPMelangeList(fname.toCharArray());
     } else {
         if (!is_daemonised)
-            std::cerr << "No ipgroups file defined in IP auth plugin config" << std::endl;
+            std::cout << "No ipgroups file defined in IP auth plugin config" << std::endl;
         syslog(LOG_ERR, "No ipgroups file defined in IP auth plugin config");
         return -1;
     }
@@ -139,7 +139,7 @@ int ipinstance::init(void *args)
 // IP-based filter group determination
 // never actually return NOUSER from this, because we don't actually look in the filtergroupslist.
 // NOUSER stops ConnectionHandler from querying subsequent plugins.
-int ipinstance::identify(Socket &peercon, Socket &proxycon, HTTPHeader &h, std::string &string)
+int ipinstance::identify(Socket &peercon, Socket &proxycon, HTTPHeader &h, std::string &string, bool &is_real_user)
 {
     // we don't get usernames out of this plugin, just a filter group
     // for now, use the IP as the username
@@ -165,12 +165,26 @@ int ipinstance::identify(Socket &peercon, Socket &proxycon, HTTPHeader &h, std::
         if (string.length() == 0)
             string = peercon.getPeerIP();
     } else {
-        string = peercon.getPeerIP();
+            string = peercon.getPeerIP();
     }
+    if (!h.header[h.header.size() - 1].find("X-Forwarded-For") == 0){
+        if (o.forwarded_for) {
+            std::string clientip;
+            clientip = peercon.getPeerIP();
+            h.addXForwardedFor(clientip); // add squid-like entry
+        }
+    }
+#ifdef DGDEBUG
+            std::cout << "IP Plugin: ------ START --------" << std::endl;
+            for (unsigned int i = 0; i < h.header.size(); i++){
+                std::cout << "IP Plugin: "<<h.header[i] << std::endl;
+            }
+            std::cout << "IP Plugin: ------ END --------" << std::endl;
+#endif
     return DGAUTH_OK;
 }
 
-int ipinstance::determineGroup(std::string &user, int &fg)
+int ipinstance::determineGroup(std::string &user, int &fg, ListContainer &uglc)
 {
     struct in_addr sin;
     inet_aton(user.c_str(), &sin);
@@ -263,7 +277,7 @@ int ipinstance::readIPMelangeList(const char *filename)
     std::ifstream input(filename);
     if (!input) {
         if (!is_daemonised) {
-            std::cerr << "Error reading file (does it exist?): " << filename << std::endl;
+            std::cout << "Error reading file (does it exist?): " << filename << std::endl;
         }
         syslog(LOG_ERR, "%s%s", "Error reading file (does it exist?): ", filename);
         return -1;
@@ -282,6 +296,7 @@ int ipinstance::readIPMelangeList(const char *filename)
     matchCIDR.comp("^[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}/[0-9]{1,2}$");
     matchRange.comp("^[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}-[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}$");
 #endif
+    RegResult Rre;
 
     // read in the file
     String line;
@@ -306,7 +321,7 @@ int ipinstance::readIPMelangeList(const char *filename)
             value = line.after("filter");
         } else {
             if (!is_daemonised)
-                std::cerr << "No filter group given; entry " << line << " in " << filename << std::endl;
+                std::cout << "No filter group given; entry " << line << " in " << filename << std::endl;
             syslog(LOG_ERR, "No filter group given; entry %s in %s", line.toCharArray(), filename);
             warn = true;
             continue;
@@ -316,19 +331,20 @@ int ipinstance::readIPMelangeList(const char *filename)
         std::cout << "value: " << value.toInteger() << std::endl;
 #endif
         if ((value.toInteger() < 1) || (value.toInteger() > o.filter_groups)) {
+	    int filtergroups = o.filter_groups;
             if (!is_daemonised)
-                std::cerr << "Filter group out of range; entry " << line << " in " << filename << std::endl;
-            syslog(LOG_ERR, "Filter group out of range; entry %s in %s", line.toCharArray(), filename);
+                std::cout << "IP plugin Filter group out of range; entry " << line << " in " << filename << " groups number: " << o.filter_groups << std::endl;
+            syslog(LOG_ERR, "IP plugin Filter group out of range; entry %s in %s groups number: %d", line.toCharArray(), filename, filtergroups);
             warn = true;
-            continue;
+            return -1;
         }
         // store the IP address (numerically, not as a string) and filter group in either the IP list, subnet list or range list
-        if (matchIP.match(key.toCharArray())) {
+        if (matchIP.match(key.toCharArray(),Rre)) {
             struct in_addr address;
             if (inet_aton(key.toCharArray(), &address)) {
                 iplist.push_back(ip(ntohl(address.s_addr), value.toInteger() - 1));
             }
-        } else if (matchSubnet.match(key.toCharArray())) {
+        } else if (matchSubnet.match(key.toCharArray(),Rre)) {
             struct in_addr address;
             struct in_addr addressmask;
             String subnet(key.before("/"));
@@ -342,7 +358,7 @@ int ipinstance::readIPMelangeList(const char *filename)
                 s.group = value.toInteger() - 1;
                 ipsubnetlist.push_back(s);
             }
-        } else if (matchCIDR.match(key.toCharArray())) {
+        } else if (matchCIDR.match(key.toCharArray(),Rre)) {
             struct in_addr address;
             struct in_addr addressmask;
             String subnet(key.before("/"));
@@ -361,7 +377,7 @@ int ipinstance::readIPMelangeList(const char *filename)
                     ipsubnetlist.push_back(s);
                 }
             }
-        } else if (matchRange.match(key.toCharArray())) {
+        } else if (matchRange.match(key.toCharArray(),Rre)) {
             struct in_addr addressstart;
             struct in_addr addressend;
             String start(key.before("-"));
@@ -377,7 +393,7 @@ int ipinstance::readIPMelangeList(const char *filename)
         // hmmm. the key didn't match any of our regular expressions. output message & return a warning value.
         else {
             if (!is_daemonised)
-                std::cerr << "Entry " << line << " in " << filename << " was not recognised as an IP address, subnet or range" << std::endl;
+                std::cout << "Entry " << line << " in " << filename << " was not recognised as an IP address, subnet or range" << std::endl;
             syslog(LOG_ERR, "Entry %s in %s was not recognised as an IP address, subnet or range", line.toCharArray(), filename);
             warn = true;
         }
