@@ -9,6 +9,7 @@
 #endif
 #include "FatController.hpp"
 #include "SysV.hpp"
+#include "Queue.hpp"
 
 #include <cstdlib>
 #include <iostream>
@@ -64,17 +65,18 @@ RegExp absurl_re, relurl_re;
 // DECLARATIONS
 
 // get the OptionContainer to read in the given configuration file
-void read_config(const char *configfile, int type);
+void read_config(std::string& configfile, int type);
 
 // IMPLEMENTATION
 
 // get the OptionContainer to read in the given configuration file
-void read_config(const char *configfile, int type)
+//void read_config(const char *configfile, int type)
+void read_config(std::string& configfile, int type)
 {
-    int rc = open(configfile, 0, O_RDONLY);
+    int rc = open(configfile.c_str(), 0, O_RDONLY);
     if (rc < 0) {
-        syslog(LOG_ERR, "Error opening %s", configfile);
-        std::cerr << "Error opening " << configfile << std::endl;
+        syslog(LOG_ERR, "Error opening %s", configfile.c_str());
+        std::cerr << "Error opening " << configfile.c_str() << std::endl;
         exit(1); // could not open conf file for reading, exit with error
     }
     close(rc);
@@ -115,28 +117,26 @@ int main(int argc, char *argv[])
                 bool dobreak = false;
                 switch (option) {
                 case 'q':
-                    read_config(configfile.c_str(), 0);
-                    return sysv_kill(o.pid_filename);
+                    read_config(configfile, 0);
+                    return sysv_kill(o.pid_filename,true);
                 case 'Q':
-                    read_config(configfile.c_str(), 0);
+                    read_config(configfile, 0);
                     sysv_kill(o.pid_filename, false);
                     // give the old process time to die
                     while (sysv_amirunning(o.pid_filename))
                         sleep(1);
                     unlink(o.pid_filename.c_str());
-                    unlink(o.ipc_filename.c_str());
-                    unlink(o.urlipc_filename.c_str());
                     // remember to reset config before continuing
                     needreset = true;
                     break;
                 case 's':
-                    read_config(configfile.c_str(), 0);
+                    read_config(configfile, 0);
                     return sysv_showpid(o.pid_filename);
                 case 'r':
-                    read_config(configfile.c_str(), 0);
+                    read_config(configfile, 0);
                     return sysv_hup(o.pid_filename);
                 case 'g':
-                    read_config(configfile.c_str(), 0);
+                    read_config(configfile, 0);
                     return sysv_usr1(o.pid_filename);
                 case 'v':
                     std::cout << "e2guardian " << PACKAGE_VERSION << std::endl
@@ -168,10 +168,9 @@ int main(int argc, char *argv[])
                     std::cout << "  -q causes e2guardian to kill any running copy." << std::endl;
                     std::cout << "  -Q kill any running copy AND start a new one with current options." << std::endl;
                     std::cout << "  -s shows the parent process PID and exits." << std::endl;
-                    std::cout << "  -r closes all connections and reloads config files by issuing a HUP," << std::endl;
-                    std::cout << "     but this does not reset the maxchildren option (amongst others)." << std::endl;
-                    std::cout << "  -g gently restarts by not closing all current connections; only reloads" << std::endl
-                              << "     filter group config files. (Issues a USR1)" << std::endl;
+                    std::cout << "  -r reloads lists and group config files by issuing a HUP," << std::endl;
+                    std::cout << "     but this does not reset the httpworkers option (amongst others)." << std::endl;
+                    std::cout << "  -g  same as -r  (Issues a USR1)" << std::endl;
                     std::cout << "  -i read total block list from stdin" << std::endl;
 #ifdef __BENCHMARK
                     std::cout << "  --bs benchmark searching filter group 1's bannedsitelist" << std::endl;
@@ -204,7 +203,7 @@ int main(int argc, char *argv[])
         o.reset();
     }
 
-    read_config(configfile.c_str(), 2);
+    read_config(configfile, 2);
 
     if ( ! o.name_suffix.empty() ) {
       prog_name += o.name_suffix;
@@ -293,7 +292,7 @@ int main(int argc, char *argv[])
             DataBuffer d(file.c_str(), file.length());
             String f;
             n.checkme(&d, f, f);
-            std::cout << n.isItNaughty << std::endl
+            std::cout << "What is naughty: "<< n.isItNaughty << std::endl
                       << n.whatIsNaughty << std::endl
                       << n.whatIsNaughtyLog << std::endl
                       << n.whatIsNaughtyCategories << std::endl;
@@ -337,28 +336,15 @@ int main(int argc, char *argv[])
     // on soft/gentle restarts headroom may be needed while children die
     // so use prefork_children as an estimate for this value.
     max_maxchildren = rlim.rlim_cur - (no_listen_fds + 6);
+    int fd_needed = (o.http_workers *2) + no_listen_fds + 6;
 
-#ifndef FD_SETSIZE_OVERIDE
-    /* Fix ugly crash */
-    /* Temporary sucurity protection about FD_SETSIZE limit - for all system now - later for no epoll system */
-
-    if (DANS_MAXFD > FD_SETSIZE) {
-        syslog(LOG_ERR, "%s", "Compiled with --with-filedescriptors too high");
-        std::cerr << " Compiled with --with-filedescriptors too high" << std::endl;
-        std::cerr << "You should upgrade your FD_SETSIZE=" << FD_SETSIZE << std::endl;
-        std::cerr << "E2guardian compiled with with-filedescriptors=" << DANS_MAXFD << std::endl;
-        std::cerr << "Or reduce --with-filedescriptors=" << DANS_MAXFD << " under " << FD_SETSIZE << std::endl;
-        return 1; // we can't have rampant proccesses can we?
-    }
-#endif
-    if ((o.max_children + o.prefork_children) > max_maxchildren) {
-        syslog(LOG_ERR, "%s", "maxchildren option in e2guardian.conf has a value too high.");
-        std::cerr << " maxchildren option in e2guardian.conf has a value too high for current file id limit (" << rlim.rlim_cur << ")" << std::endl;
-        std::cerr << "The total of maxchildren " << o.max_children << " plus preforkchildren " << o.prefork_children << " must not exceed " << max_maxchildren << "" << std::endl;
+    if (((o.http_workers * 2) ) > max_maxchildren) {
+        syslog(LOG_ERR, "%s", "httpworkers option in e2guardian.conf has a value too high.");
+        std::cerr << " httpworkers option in e2guardian.conf has a value too high for current file id limit (" << rlim.rlim_cur << ")" << std::endl;
+        std::cerr << "httpworkers " << o.http_workers <<  " must not exceed 50% of " << max_maxchildren << "" << std::endl;
         std::cerr << "in this configuration." << std::endl;
-        std::cerr << "Reduce maxchildren and/or preforkchilden" << std::endl;
-        std::cerr << "Or recompile with an increased --with-filedescriptors=" << DANS_MAXFD << " and/or " << std::endl;
-        std::cerr << "upgrade your FD_SETSIZE=" << FD_SETSIZE << std::endl;
+        std::cerr << "Reduce httpworkers " << std::endl;
+        std::cerr << "Or increase the filedescriptors available with ulimit -n to at least=" << fd_needed << std::endl;
         return 1; // we can't have rampant proccesses can we?
     }
 
@@ -462,7 +448,7 @@ int main(int argc, char *argv[])
             std::cout << "About to re-read conf file." << std::endl;
 #endif
             o.reset();
-            if (!o.read(configfile.c_str(), 2)) {
+            if (!o.read(configfile, 2)) {
                 syslog(LOG_ERR, "%s", "Error re-parsing the e2guardian.conf file or other e2guardian configuration files");
 #ifdef DGDEBUG
                 std::cerr << "Error re-parsing the e2guardian.conf file or other e2guardian configuration files" << std::endl;
